/* time.c - time the fps of a system
**
** Copyright (C) 1995  Jonathan Paul Griffiths.  All rights reserved.
**
** You may do anything with this code you wish EXCEPT sell it. You may sell
** any software you create using this code,  but you MUST NOT charge for
** the code itself.  See the file "jlib.doc" for more details.
*/

/* I'm not sure who supports gettimeofday and who doesn't */
/* but I have a feeling its a GNU only thing */

#include <sys/time.h>
#include <unistd.h>
#include <jlib.h>

/* this routine comes from the nice GNU folks.. */
int jtimeval_subtract (result, x, y)
 struct timeval *result, *x, *y;
 {
   /* Perform the carry for the later subtraction by updating y. */
   if (x->tv_usec < y->tv_usec) {
      int nsec = (y->tv_usec - x->tv_usec) / 1000000 + 1;
      y->tv_usec -= 1000000 * nsec;
      y->tv_sec += nsec;
   }
   if (x->tv_usec - y->tv_usec > 1000000) {
      int nsec = (y->tv_usec - x->tv_usec) / 1000000;
      y->tv_usec += 1000000 * nsec;
      y->tv_sec -= nsec;
   }
   /* Compute the time remaining to wait.
      tv_usec' is certainly positive. */
   result->tv_sec = x->tv_sec - y->tv_sec;
   result->tv_usec = x->tv_usec - y->tv_usec;
                                                                                              
   /* Return 1 if result is negative. */
   return x->tv_sec < y->tv_sec;
 }
 

/* return the current frames per second blitting a full screen buffer */
double jlib_get_fps(int num_iters)
{
 buffer_rec *buff;
 struct timeval start_time,end_time,final_time;
 int i; 
 double fps;
 
 buff = buff_init(SCREEN_WIDTH,SCREEN_HEIGHT);

 gettimeofday(&start_time,NULL);
 
 for(i=0;i<num_iters;i++){
     screen_blit_fs_buffer(buff);
 }  
 
 gettimeofday(&end_time,NULL);

 buff_free(buff);

 jtimeval_subtract(&final_time,&end_time,&start_time); 

 fps = (double)num_iters/(double)((double)final_time.tv_usec+((double)final_time.tv_sec*1000000.0));
 fps = fps*1000000.0; 
  

 return fps;
}


