/*
   ** Copyright (C) 1995  Jonathan Paul Griffiths.  All rights reserved.
   **
   ** You may do anything with this code you wish EXCEPT sell it. You may sell
   ** any software you create using this code,  but you MUST NOT charge for
   ** the code itself.  See the file "jlib.doc" for detailed terms. 
 */
#include <stdlib.h>
#include <jlib.h>

/* private function */
void masked_copy(UBYTE *s,UBYTE *d, unsigned int len);

/*+------------------------------------------------------------------------+ */
/*|blit a portion of a buffer to another buffer without clipping           | */
/*+------------------------------------------------------------------------+ */
JINLINE void buff_blit_buff_toNC (buffer_rec * dbuff, int dbfx, int dbfy,
     buffer_rec * sbuff, int sbx1, int sby1, int sbx2, int sby2)
{
   int v_iterations;
   unsigned int h_len;

   UBYTE *src = B_BUFF_PTR (sbuff), *dest = B_BUFF_PTR (dbuff);
   int sbuffx = B_X_SIZE (sbuff), dbuffx = B_X_SIZE (dbuff);

   JLIB_PRINT_DEBUG_INFO ("Entering buff_blit_buff_toNC()\n");

   /* sort coordinates if needed */
   if (sbx2 < sbx1) {
      JLIB_SWAP(sbx1, sbx2);
   }
   if (sby2 < sby1) {
      JLIB_SWAP(sby1, sby2);
   }

   src += (sby1 *  sbuffx) + sbx1;
   dest += (dbfy * dbuffx) + dbfx;

   v_iterations = (sby2 - sby1)+1;        /* number of times to repeat horiz. blit */
   h_len = (sbx2 - sbx1)+1;       /* length of horizontal blit */

   switch (get_best_copy_method ((void *) src, (void *) dest, (unsigned int) h_len)) {

   case ALIGNMENT_STATE_1:
      for (; v_iterations != 0; v_iterations--) {
	 MEM_COPY_ALIGN_1 (src, dest, h_len);
	 src += sbuffx;
	 dest += dbuffx;
      }
      break;

   case ALIGNMENT_STATE_2:
      for (; v_iterations != 0; v_iterations--) {
	 MEM_COPY_ALIGN_2 (src, dest, h_len);
	 src += sbuffx;
	 dest += dbuffx;
      }
      break;

   case ALIGNMENT_STATE_3:
      for (; v_iterations != 0; v_iterations--) {
	 MEM_COPY_ALIGN_3 (src, dest, h_len);
	 src += sbuffx;
	 dest += dbuffx;
      }
      break;

   case ALIGNMENT_STATE_4:
      for (; v_iterations != 0; v_iterations--) {
	 MEM_COPY_ALIGN_4 (src, dest, h_len);
	 src += sbuffx;
	 dest += dbuffx;
      }
      break;

   case ALIGNMENT_STATE_5:
      for (; v_iterations != 0; v_iterations--) {
	 MEM_COPY_ALIGN_5 (src, dest, h_len);
	 src += sbuffx;
	 dest += dbuffx;
      }
      break;

   case ALIGNMENT_STATE_6:
      for (; v_iterations != 0; v_iterations--) {
	 MEM_COPY_ALIGN_6 (src, dest, h_len);
	 src += sbuffx;
	 dest += dbuffx;
      }
      break;

   case ALIGNMENT_STATE_7:
      for (; v_iterations != 0; v_iterations--) {
	 MEM_COPY_ALIGN_7 (src, dest, h_len);
	 src += sbuffx;
	 dest += dbuffx;
      }
      break;

   case ALIGNMENT_STATE_8:
      for (; v_iterations != 0; v_iterations--) {
	 MEM_COPY_ALIGN_8 (src, dest, h_len);
	 src += sbuffx;
	 dest += dbuffx;
      }
      break;

   default:
      jlib_exit("Unknown value returned from get_best_copy_method().");
      break;
   }

   JLIB_PRINT_DEBUG_INFO ("Leaving buff_blit_buff_toNC()\n");
}

/*+------------------------------------------------------------------------+ */
/*|blit a portion of a buffer to another buffer with clipping              | */
/*+------------------------------------------------------------------------+ */
JINLINE void buff_blit_buff_to (buffer_rec * dbuff, int dbfx, int dbfy,
     buffer_rec * sbuff, int sbx1, int sby1, int sbx2, int sby2)
{
   int v_iterations;
   unsigned int h_len;
   UBYTE *src,*dest;
   int sbuffx,sbuffy,dbuffx,dbuffy;

   JLIB_PRINT_DEBUG_INFO ("Entering buff_blit_buff_to()\n");

   if((sbuff == NULL) || (dbuff == NULL)){
      jlib_exit("NULL buffer passed to buff_blit_buff_to");
   }

   src = B_BUFF_PTR (sbuff);
   dest = B_BUFF_PTR (dbuff);
   sbuffx = B_X_SIZE (sbuff);
   sbuffy = B_Y_SIZE(sbuff);
   dbuffx = B_X_SIZE (dbuff);
   dbuffy = B_Y_SIZE (dbuff);

   /* sort coordinates if needed */
   if (sbx2 < sbx1) {
      JLIB_SWAP(sbx1, sbx2);
   }
   if (sby2 < sby1) {
      JLIB_SWAP(sby1, sby2);
   }

   /* test dest coordinates for "don't draw" clipping */
   if ((dbfx >= dbuffx) || (dbfy >= dbuffy)) {
      return;
   }

   if(dbfx < 0){
      sbx1 += (-dbfx);
      dbfx = 0;
      if(sbx1 > sbx2){
	return;
      }
   }
   
   if(dbfy < 0){
      sby1 += (-dbfy);
      dbfy = 0;
      if(sby1 > sby2){
	 return;
      }
   }

   /* test source coordinates for "don't draw" clipping */
   if ((sbx1 >= sbuffx) || (sby1 >= sbuffy)) {
      return;
   }

   if ((sbx2 < 0) || (sby2 <0)) {
      return;
   }

   /* clip source points to the source buffer */
   if(sbx1 < 0){
      sbx1 = 0;
   }
   if(sby1 < 0){
      sby1 = 0;
   }
   if(sbx2 >= sbuffx){
      sbx2 = sbuffx-1;
   }
   if(sby2 >= sbuffy){
      sby2 = sbuffy-1;
   }

   src += (sby1 * sbuffx) + sbx1;
   dest += (dbfy * dbuffx) + dbfx;

   v_iterations = (sby2 - sby1)+1; /* number of times to repeat horiz. blit */
   h_len = (sbx2 - sbx1)+1;        /* length of horizontal blit */

   /* test coordinates to clip area drawn */
   if (dbfx + h_len >= dbuffx) {
      h_len = dbuffx - dbfx;
   }
   if (dbfy + v_iterations >= dbuffy) {
      v_iterations = dbuffy - dbfy;
   }

   switch (get_best_copy_method ((void *) src, (void *) dest, (unsigned int) h_len)) {

   case ALIGNMENT_STATE_1:
      for (; v_iterations != 0; v_iterations--) {
	 MEM_COPY_ALIGN_1 (src, dest, h_len);
	 src += sbuffx;
	 dest += dbuffx;
      }
      break;

   case ALIGNMENT_STATE_2:
      for (; v_iterations != 0; v_iterations--) {
	 MEM_COPY_ALIGN_2 (src, dest, h_len);
	 src += sbuffx;
	 dest += dbuffx;
      }
      break;

   case ALIGNMENT_STATE_3:
      for (; v_iterations != 0; v_iterations--) {
	 MEM_COPY_ALIGN_3 (src, dest, h_len);
	 src += sbuffx;
	 dest += dbuffx;
      }
      break;

   case ALIGNMENT_STATE_4:
      for (; v_iterations != 0; v_iterations--) {
	 MEM_COPY_ALIGN_4 (src, dest, h_len);
	 src += sbuffx;
	 dest += dbuffx;
      }
      break;

   case ALIGNMENT_STATE_5:
      for (; v_iterations != 0; v_iterations--) {
	 MEM_COPY_ALIGN_5 (src, dest, h_len);
	 src += sbuffx;
	 dest += dbuffx;
      }
      break;

   case ALIGNMENT_STATE_6:
      for (; v_iterations != 0; v_iterations--) {
	 MEM_COPY_ALIGN_6 (src, dest, h_len);
	 src += sbuffx;
	 dest += dbuffx;
      }
      break;

   case ALIGNMENT_STATE_7:
      for (; v_iterations != 0; v_iterations--) {
	 MEM_COPY_ALIGN_7 (src, dest, h_len);
	 src += sbuffx;
	 dest += dbuffx;
      }
      break;

   case ALIGNMENT_STATE_8:
      for (; v_iterations != 0; v_iterations--) {
	 MEM_COPY_ALIGN_8 (src, dest, h_len);
	 src += sbuffx;
	 dest += dbuffx;
      }
      break;

   default:
      jlib_exit("Unknown value returned from get_best_copy_method()");
      break;
   }

   JLIB_PRINT_DEBUG_INFO ("Leaving buff_blit_buff_to()\n");
}

/*+------------------------------------------------------------------------+ */
/*|blit a portion of an image to a buffer with clipping                    | */
/*+------------------------------------------------------------------------+ */

JINLINE void buff_blit_img_to (buffer_rec * dbuff, int dbfx, int dbfy,
	    image * img, int imx1, int imy1, int imx2, int imy2)
{
   buffer_rec fake_buff;

   JLIB_PRINT_DEBUG_INFO ("Entering buff_blit_img_to()\n");

   if((img == NULL) || (dbuff == NULL)){
      jlib_exit("NULL pointer passed to buff_blit_img_to");
   }

   /* fake the image -> buffer copy as a buff to buff copy */
   B_X_SIZE(&fake_buff)   = IMG_WIDTH(img);
   B_Y_SIZE(&fake_buff)   = IMG_HEIGHT(img);
   B_BUFF_PTR(&fake_buff) = IMG_DATA_PTR(img);

   /* do blit */
   buff_blit_buff_to (dbuff, dbfx, dbfy, &fake_buff, imx1, imy1, imx2, imy2);

   JLIB_PRINT_DEBUG_INFO ("Leaving buff_blit_img_to()\n");
}

/*+------------------------------------------------------------------------+ */
/*|blit a portion of an image to a buffer without clipping                 | */
/*+------------------------------------------------------------------------+ */

JINLINE void buff_blit_img_toNC (buffer_rec * dbuff, int dbfx, int dbfy,
	    image * img, int imx1, int imy1, int imx2, int imy2)
{
   buffer_rec fake_buff;

   JLIB_PRINT_DEBUG_INFO ("Entering buff_blit_img_toNC()\n");

   /* fake the image -> buffer copy as a buff to buff copy */
   B_X_SIZE(&fake_buff)   = IMG_WIDTH(img);
   B_Y_SIZE(&fake_buff)   = IMG_HEIGHT(img);
   B_BUFF_PTR(&fake_buff) = IMG_DATA_PTR(img);

   /* do blit */
   buff_blit_buff_toNC (dbuff, dbfx, dbfy, &fake_buff, imx1, imy1, imx2, imy2);


   JLIB_PRINT_DEBUG_INFO ("Leaving buff_blit_img_toNC()\n");
}


JINLINE void masked_copy(UBYTE *s,UBYTE *d, unsigned int len)
{
 while(len != 0){
    if(*s != 0){
       *d = *s; /* store non-zero pixels */
    }
    ++s;
    ++d;
    --len;
 }
}


/*+------------------------------------------------------------------------+ */
/*|blit a masked portion of a buffer to another buffer without clipping    | */
/*+------------------------------------------------------------------------+ */
JINLINE void buff_stencil_buff_toNC (buffer_rec * dbuff, int dbfx, int dbfy,
     buffer_rec * sbuff, int sbx1, int sby1, int sbx2, int sby2)
{
   int v_iterations;
   unsigned int h_len;

   UBYTE *src = B_BUFF_PTR (sbuff), *dest = B_BUFF_PTR (dbuff);
   int sbuffx = B_X_SIZE (sbuff), dbuffx = B_X_SIZE (dbuff);

   JLIB_PRINT_DEBUG_INFO ("Entering buff_stencil_buff_toNC()\n");

   /* sort coordinates if needed */
   if (sbx2 < sbx1) {
      JLIB_SWAP(sbx1, sbx2);
   }
   if (sby2 < sby1) {
      JLIB_SWAP(sby1, sby2);
   }

   src += (sby1 *  sbuffx) + sbx1;
   dest += (dbfy * dbuffx) + dbfx;

   v_iterations = (sby2 - sby1)+1; /* vertical length horiz. blit */
   h_len = (sbx2 - sbx1)+1;        /* length of horizontal blit */

   for(; v_iterations != 0; v_iterations--){
      masked_copy(src, dest, h_len);
      src += sbuffx;
      dest += dbuffx;
   }

   JLIB_PRINT_DEBUG_INFO ("Leaving buff_stencil_buff_toNC()\n");
}


/*+------------------------------------------------------------------------+ */
/*|blit a masked portion of a buffer to another buffer with clipping       | */
/*+------------------------------------------------------------------------+ */
JINLINE void buff_stencil_buff_to (buffer_rec * dbuff, int dbfx, int dbfy,
     buffer_rec * sbuff, int sbx1, int sby1, int sbx2, int sby2)
{
   int v_iterations;
   unsigned int h_len;
   UBYTE *src,*dest;
   int sbuffx,sbuffy,dbuffx,dbuffy;

   JLIB_PRINT_DEBUG_INFO ("Entering buff_stencil_buff_to()\n");

   if((sbuff == NULL) || (dbuff == NULL)){
      jlib_exit("NULL buffer passed to buff_stencil_buff_to");
   }

   src = B_BUFF_PTR (sbuff);
   dest = B_BUFF_PTR (dbuff);
   sbuffx = B_X_SIZE (sbuff);
   sbuffy = B_Y_SIZE(sbuff);
   dbuffx = B_X_SIZE (dbuff);
   dbuffy = B_Y_SIZE (dbuff);

   /* sort coordinates if needed */
   if (sbx2 < sbx1) {
      JLIB_SWAP(sbx1, sbx2);
   }
   if (sby2 < sby1) {
      JLIB_SWAP(sby1, sby2);
   }

   /* test dest coordinates for "don't draw" clipping */
   if ((dbfx >= dbuffx) || (dbfy >= dbuffy)) {
      return;
   }

   if(dbfx < 0){
      sbx1 += (-dbfx);
      dbfx = 0;
      if(sbx1 > sbx2){
	return;
      }
   }
   
   if(dbfy < 0){
      sby1 += (-dbfy);
      dbfy = 0;
      if(sby1 > sby2){
	 return;
      }
   }


   /* test source coordinates for "don't draw" clipping */
   if ((sbx1 >= sbuffx) || (sby1 >= sbuffy)) {
      return;
   }

   if ((sbx2 < 0) || (sby2 <0)) {
      return;
   }

   /* clip source points to the source buffer */
   if(sbx1 < 0){
      sbx1 = 0;
   }
   if(sby1 < 0){
      sby1 = 0;
   }
   if(sbx2 >= sbuffx){
      sbx2 = sbuffx-1;
   }
   if(sby2 >= sbuffy){
      sby2 = sbuffy-1;
   }

   src += (sby1 * sbuffx) + sbx1;
   dest += (dbfy * dbuffx) + dbfx;

   v_iterations = (sby2 - sby1)+1; /* number of times to repeat horiz. blit */
   h_len = (sbx2 - sbx1)+1;        /* length of horizontal blit */

   /* test coordinates to clip area drawn */
   if (dbfx + h_len >= dbuffx) {
      h_len = dbuffx - dbfx;
   }
   if (dbfy + v_iterations >= dbuffy) {
      v_iterations = dbuffy - dbfy;
   }

   for (; v_iterations != 0; v_iterations--) {
	 masked_copy(src, dest, h_len);
	 src += sbuffx;
	 dest += dbuffx;
   }

   JLIB_PRINT_DEBUG_INFO ("Leaving buff_stencil_buff_to()\n");
}
