/*
   ** Copyright (C) 1995  Jonathan Paul Griffiths.  All rights reserved.
   **
   ** You may do anything with this code you wish EXCEPT sell it. You may sell
   ** any software you create using this code,  but you MUST NOT charge for
   ** the code itself.  See the file "jlib.doc" for detailed terms. 
 */
#include <string.h>
#include <jlib.h>

/* clipping codes */
#define C_LEFT (1<<0)
#define C_RIGHT (1<<1)
#define C_UP (1<<2)
#define C_DOWN (1<<3)

/*+------------------------------------------------------------------------+ */
/*|draw a box with corners x1, y1 to x2 , y2 in color c with clipping      | */
/*+------------------------------------------------------------------------+ */

JINLINE void buff_draw_box (buffer_rec * buff, int x1, int y1, int x2, int y2, UBYTE c)
{
   UBYTE *buffer;
   int width;
   int height;
   int boxwidth;
   unsigned char status = 0;
   int temp;
   
   JLIB_PRINT_DEBUG_INFO ("Entering buff_draw_box()\n");

   if(buff == NULL){
      jlib_exit("NULL buffer passed to buff_draw_box.");
   }
   
   buffer = B_BUFF_PTR (buff);
   width = B_X_SIZE (buff);
   height = B_Y_SIZE (buff);

   if (x1 > x2) {
      x2 ^= x1;
      x1 ^= x2;			/* swap ends around */
      x2 ^= x1;
   }

   if (y1 > y2) {
      y2 ^= y1;
      y1 ^= y2;			/* swap ends around */
      y2 ^= y1;
   }

   /* clip coords if nessisary */
   if ((x1 >= width) || (y1 >= height) || (x2 < 0 ) || (y2 < 0)) {
      return;
   }

   if (x2 >= width) {
      x2 = width-1;
      status |= C_RIGHT;
   }

   if (y2 >= height) {
      y2 = height-1;
      status |= C_DOWN;
   }
   
   if (x1 < 0) {
      x1 = 0;
      status |= C_LEFT;
   }
   
   if (y1 < 0) {
      y1 = 0;
      status |= C_UP;
   }
   
   /* use the clipping status to determine which lines to draw */
   if(status == 0){
      boxwidth = x2 - x1;
      y2 -= y1;

     /* draw top line */
     buffer += ((unsigned int)width * (unsigned int) y1);
     buffer += x1;
     memset (buffer, (int) c, (size_t) boxwidth + 1);	/* this is NOT a cludge! */

     /* draw the 2 vertical sides at the same tip */
     for (; y2 != 0; y2--) {
        buffer += width;
        *buffer = c;
        *(buffer + boxwidth) = c;
     }

     /* draw bottom line  */
     memset (buffer, (int) c, (size_t) boxwidth);
     return;
   }
 
   if((status&C_UP)==0){
      /* draw top line */
      buff_draw_h_lineNC(buff,x1,y1,x2,c);
   }
   
   if((status&C_DOWN)==0){
      /* draw bottom line */
      buff_draw_h_lineNC(buff,x1,y2,x2,c);
   }

   if((status&C_LEFT)==0){
      /* draw left line */
      buffer=B_BUFF_PTR(buff)+ (width * y1) + x1;

      for (temp = y2-y1; temp >= 0; temp--) {
         *buffer = c;
         buffer += width;
      }
      
   }

   if((status&C_RIGHT)==0){
      /* draw right line */
      buffer = B_BUFF_PTR(buff)+ (width * y1) + x2;

      for (temp = y2-y1; temp >= 0; temp--) {
         *buffer = c;
         buffer += width;
      }

    }
      
   JLIB_PRINT_DEBUG_INFO ("Leaving buff_draw_box()\n");
}


/*+------------------------------------------------------------------------+ */
/*|draw a box with corners x1, y1 to x2 , y2 in color c without clipping   | */
/*+------------------------------------------------------------------------+ */

JINLINE void buff_draw_boxNC (buffer_rec * buff, int x1, int y1, int x2, int y2, UBYTE c)
{
   UBYTE *buffer = B_BUFF_PTR (buff);
   int width = B_X_SIZE (buff);
   int boxwidth;

   JLIB_PRINT_DEBUG_INFO ("Entering buff_draw_box()\n");

   if (x1 > x2) {
      x2 ^= x1;
      x1 ^= x2;			/* swap ends around */
      x2 ^= x1;
   }

   if (y1 > y2) {
      y2 ^= y1;
      y1 ^= y2;			/* swap ends around */
      y2 ^= y1;
   }

   boxwidth = x2 - x1;
   y2 -= y1;

   /* draw top line */
   buffer += ((unsigned int) width * (unsigned int) y1);
   buffer += x1;
   memset (buffer, (int) c, (size_t) boxwidth + 1);	/* this is NOT a cludge! */

   /* draw the 2 vertical sides at the same tip */
   for (; y2 != 0; y2--) {
      buffer += width;
      *buffer = c;
      *(buffer + boxwidth) = c;
   }

   /* draw bottom line  */
   memset (buffer, (int) c, (size_t) boxwidth);

   JLIB_PRINT_DEBUG_INFO ("Leaving buff_draw_box()\n");
}
