/*
   ** Copyright (C) 1995  Jonathan Paul Griffiths.  All rights reserved.
   **
   ** You may do anything with this code you wish EXCEPT sell it. You may sell
   ** any software you create using this code,  but you MUST NOT charge for
   ** the code itself.  See the file "jlib.doc" for detailed terms. 
 */
#include <string.h>
#include <jlib.h>

#define ABS(a)   ((a < 0) ? -a : a)
#define SGN(a)   ((a < 0) ? -1 : 1)

/*+-------------------------------------------------------------------------+ */
/*|draw a line from x1, y1 to x2, y2 in color c                             | */
/*|                                                                         | */
/*+-------------------------------------------------------------------------+ */
void buff_draw_line (buffer_rec * buff, int x1, int y1, int x2, int y2, UBYTE c)
{
   int d, x, y, ax, ay, sx, sy, dx, dy;
   UBYTE *buffer;
   int width,max_x,max_y,delta;
   
   JLIB_PRINT_DEBUG_INFO ("Entering buff_draw_line()\n");

   if(buff == NULL){
      jlib_exit("NULL buffer passed to buff_draw_line.");
   }

   buffer = B_BUFF_PTR (buff);
   width = B_X_SIZE (buff);
   max_y = B_Y_SIZE (buff)-1;
   max_x = width -1;

   if(x1 > x2){
     JLIB_SWAP(x1,x2);
     JLIB_SWAP(y1,y2);
   }                    
    
   if((x2 < 0) || (x1 > max_x)){
     return;
   }
						
   if(y1 < y2){
							
      if((y2 < 0) || (y1 > max_y)) {            
	 return;                                                
      }                                                         
      if(y1 < 0) {                                      
	 delta = (x2 - x1)*(0 - y1)/(y2 - y1);  
	 if((x1 += delta) > max_x){
	    return;
	 }              
	 y1 = 0;                                                
       }                                                                
       if(y2 > max_y) {                                 
	  delta = (x2 - x1)*(y2 - max_y)/(y2 - y1);     

	  if((x2 -= delta) < 0){
	     return;
	  }             
	  y2 = max_y;                                           
       }                                                                
       if(x1 < 0) {                                     
	  delta = (y2 - y1)*(0 - x1)/(x2 - x1); 
	  y1 += delta;                                                  
	  x1 = 0;                                               
       }                                                                
       if(x2 > max_x) {                                 
	  delta = (y2 - y1)*(x2 - max_x)/(x2 - x1);     
	  y2 -= delta;                                                  
	  x2 = max_x;                                           
       }                                                                
   }                                                                    
   else{                                                                
       if((y1 < 0) || (y2 > max_y)) {           
	  return;                                               
       }                                                                
       if(y1 > max_y) {                                 
	  delta = (x2 - x1)*(y1 - max_y)/(y1 - y2);     
	  if((x1 += delta) > max_x) { return; }         
	  y1 = max_y;                                           
       }                                                                
       if(y2 < 0) {                                     
	  delta = (x2 - x1)*(0 - y2)/(y1 - y2); 
	  if((x2 -= delta) < 0) { return; }             
	  y2 = 0;                                               
       }                                                                
       if(x1 < 0) {                                     
	  delta = (y1 - y2)*(0 - x1)/(x2 - x1); 
	  y1 -= delta;                                                  
	  x1 = 0;                                               
       }                                                                
       if(x2 > max_x) {                                 
	  delta = (y1 - y2)*(x2 - max_x)/(x2 - x1);     
	  y2 += delta;                                                  
	  x2 = max_x;                                           
       }                                                                
   }                                                                    

   dx = x2 - x1;
   ax = ABS (dx) << 1;
   sx = SGN (dx);
   dy = y2 - y1;
   ay = ABS (dy) << 1;
   sy = SGN (dy);

   x = x1;
   y = y1;

   if (ax > ay) {
      d = ay - (ax >> 1);
      while (x != x2) {
	 buffer[(width * y) + x] = c;
	 if (d >= 0) {
	    y += sy;
	    d -= ax;
	 }
	 x += sx;
	 d += ay;
      }
   }
   else {
      d = ax - (ay >> 1);
      while (y != y2) {
	 buffer[(width * y) + x] = c;
	 if (d >= 0) {
	    x += sx;
	    d -= ay;
	 }
	 y += sy;
	 d += ax;
      }
   }

   JLIB_PRINT_DEBUG_INFO ("Leaving buff_draw_line()\n");
}

/*+-------------------------------------------------------------------------+ */
/*|draw a line from x1, y1 to x2, y2 in color c                             | */
/*+-------------------------------------------------------------------------+ */

void buff_draw_lineNC (buffer_rec * buff, int x1, int y1, int x2, int y2, UBYTE c)
{
   int d, x, y, ax, ay, sx, sy, dx, dy;
   UBYTE *buffer = B_BUFF_PTR (buff);
   int width = B_X_SIZE (buff);

   JLIB_PRINT_DEBUG_INFO ("Entering buff_draw_line_NC()\n");

   dx = x2 - x1;
   ax = ABS (dx) << 1;
   sx = SGN (dx);
   dy = y2 - y1;
   ay = ABS (dy) << 1;
   sy = SGN (dy);

   x = x1;
   y = y1;

   if (ax > ay) {
      d = ay - (ax >> 1);
      while (x != x2) {
	 buffer[(width * y) + x] = c;
	 if (d >= 0) {
	    y += sy;
	    d -= ax;
	 }
	 x += sx;
	 d += ay;
      }
   }
   else {
      d = ax - (ay >> 1);
      while (y != y2) {
	 buffer[(width * y) + x] = c;
	 if (d >= 0) {
	    x += sx;
	    d -= ay;
	 }
	 y += sy;
	 d += ax;
      }
   }

   JLIB_PRINT_DEBUG_INFO ("Leaving buff_draw_line_NC()\n");
}


/*+------------------------------------------------------------------------+ */
/*|draw a horizontal line from x1, y1 to x2 in color c with clipping       | */
/*+------------------------------------------------------------------------+ */

void buff_draw_h_line (buffer_rec * buff, int x1, int y1, int x2, UBYTE c)
{
   UBYTE *buffer;
   int width,height;

   JLIB_PRINT_DEBUG_INFO ("Entering buff_draw_h_line()\n");

   if(buff == NULL){
      jlib_exit("NULL buffer passed to buff_draw_h_line.");
   }

   buffer = B_BUFF_PTR (buff);
   width = B_X_SIZE (buff);
   height = B_Y_SIZE (buff);

   if (x1 > x2) {
      JLIB_SWAP(x2,x1);
   }

   /* clip points */
   if ((x1 >= width) || (y1 >= height) || (x2 < 0) || (y1 < 0)) {
      return;
   }

   if (x2 >= width) {
      x2 = width-1;
   }
   
   if (x1 <0){
      x1 =0;
   }

   buffer += (width * y1) + x1;
   memset (buffer, (int) c, (size_t) (x2 - x1)+1); 

   JLIB_PRINT_DEBUG_INFO ("Leaving buff_draw_h_line()\n");
}

/*+------------------------------------------------------------------------+ */
/*|draw a horizontal line from x1, y1 to x2 in color c without clipping    | */
/*+------------------------------------------------------------------------+ */

void buff_draw_h_lineNC (buffer_rec * buff, int x1, int y1, int x2, UBYTE c)
{
   UBYTE *buffer = B_BUFF_PTR (buff);
   int width = B_X_SIZE (buff);

   JLIB_PRINT_DEBUG_INFO ("Entering buff_draw_h_line_NC()\n");

   if (x1 > x2) {
     JLIB_SWAP(x2,x1);
   }

   buffer += (width * y1) + x1;

   memset (buffer, (int) c, (size_t) (x2 - x1)+1 ); /* +1 not a mistake */

   JLIB_PRINT_DEBUG_INFO ("Leaving buff_draw_h_line_NC()\n");
}


/*+------------------------------------------------------------------------+ */
/*|draw a vertical line from x1, y1 to x2 in color c with clipping         | */
/*+------------------------------------------------------------------------+ */

void buff_draw_v_line (buffer_rec * buff, int x1, int y1, int y2, UBYTE c)
{
   UBYTE *buffer = B_BUFF_PTR (buff);
   int width = B_X_SIZE (buff);
   int height = B_Y_SIZE (buff);

   JLIB_PRINT_DEBUG_INFO ("Entering buff_draw_v_line()\n");

   if(buff == NULL){
      jlib_exit("NULL buffer passed to buff_draw_v_line.");
   }

   if (y1 > y2) {
     JLIB_SWAP(y2,y1);
   }

   /* clip coords if needed */
   if ((y1 >= height) || (x1 >= width) || (y2 < 0) || (x1 < 0)) {
      return;
   }

   if (y2 >= height) {
      y2 = height-1;
   }

   if (y1 <0){
      y1 = 0;
   }
   
   buffer += (width * y1) + x1;

   for (; y2 >= y1; --y2) {
      *buffer = c;
      buffer += width;
   }

   JLIB_PRINT_DEBUG_INFO ("Leaving buff_draw_v_line()\n");
}


/*+------------------------------------------------------------------------+ */
/*|draw a vertical line from x1, y1 to x2 in color c without clipping      | */
/*+------------------------------------------------------------------------+ */

void buff_draw_v_lineNC (buffer_rec * buff, int x1, int y1, int y2, UBYTE c)
{
   UBYTE *buffer = B_BUFF_PTR (buff);
   int width = B_X_SIZE (buff);

   JLIB_PRINT_DEBUG_INFO ("Entering buff_draw_v_lineNC()\n");

   if (y1 > y2) {
     JLIB_SWAP(y2,y1);
   }

   buffer += (width * y1) + x1;

   for (; y2 >= y1; --y2) {
      *buffer = c;
      buffer += width;
   }

   JLIB_PRINT_DEBUG_INFO ("Leaving buff_draw_v_lineNC()\n");
}
