/*
   ** Copyright (C) 1995  Jonathan Paul Griffiths.  All rights reserved.
   **
   ** You may do anything with this code you wish EXCEPT sell it. You may sell
   ** any software you create using this code,  but you MUST NOT charge for
   ** the code itself.  See the file "jlib.doc" for detailed terms. 
 */

#include <string.h>
#include <jlib.h>

#define NOT_CLIPPED 0

#define LEFT   (xpos)
#define RIGHT  (xpos+width)
#define TOP    (ypos)
#define BOT (ypos+height)

/*+----------------------------------------------------------------------+ */
/*|draw_sprite - draw a sprite on screen with clipping                   | */
/*+----------------------------------------------------------------------+ */

void buff_draw_sprite (sprite_system * spr_sys, int snum, buffer_rec * obuff)
{
   sprite_data_rec *current_sprite_data;
   int  xpos, ypos,bwidth,bheight;
   UBYTE *data,width, height,count1, count2, endx, endy;
   UBYTE startx,starty;
   UBYTE status = NOT_CLIPPED;
   UBYTE *pos_in_buff, datum;

   JLIB_PRINT_DEBUG_INFO ("Entering buff_draw_sprite()\n");

   if((obuff == NULL)||(spr_sys==NULL)){
      jlib_exit("NULL handle passed to buff_draw_sprite.");
   }
   
   bwidth  = B_X_SIZE(obuff);
   bheight = B_Y_SIZE(obuff);
   
   /* don't draw sprite if it isn't turned on */
   if (SPR_IS_ON (spr_sys, snum)) {

      /* the data is the sprite_data_rec indicated by it's frame */
      current_sprite_data = spr_sys->sprite_data[spr_sys->sprites[snum]->frame];

      if(current_sprite_data == NULL){
         jlib_exit("NULL sprite_data_rec passed to buff_draw_sprite.");
      }

      data = current_sprite_data->data;
      width = current_sprite_data->width;
      height = current_sprite_data->height;

      /* set initial values of startx , endx etc */
      startx = starty = 0;
      endx = width;
      endy = height;

      xpos = spr_sys->sprites[snum]->x;
      ypos = spr_sys->sprites[snum]->y;


      /* check for "don't draw"  clipping  */
      if ((LEFT >= bwidth)||(RIGHT<0)||(TOP >=bheight)||(BOT < 0)){
	 return;
      }

      /* check the other 4 clipping conditions */
      if (LEFT < 0) {
	 /* some portion of the left side of the sprite is obscured */
	 status++;
	 startx = - LEFT;
      }
      else {
	 if (RIGHT >= bwidth) {
	    /* some portion of the right side of the sprite is obscured */
	    status++;
	    endx = bwidth - LEFT;
	 }
      }

      if (TOP < 0) {
	 /* some portion of the top side of the sprite is obscured */
	 status++;
	 starty = - TOP;
      }
      else {
	 if (BOT >= bheight) {
	    /* some portion of the bottom side of the sprite is obscured */
	    status++;
	    endy = bheight - TOP;
	 }
      }

      /* only use the drawing routine below if needed */
      if (status == NOT_CLIPPED) {
	 buff_draw_spriteNC (spr_sys, snum, obuff);
	 return;
      }


      pos_in_buff = B_BUFF_PTR(obuff)+(ypos * bwidth)+ xpos;

      /* start at the appropriate place in the data */
      data += (starty * width) + startx;

      /* start at the appropriate place in buff */
      pos_in_buff += (starty * bwidth) + startx;

      /* move through the data, determining which pixels to draw */
      for (count1 = starty; count1 < endy; count1++) {
	 for (count2 = startx; count2 < endx; count2++) {
	    datum = *data;

	    if (datum) {
	       *pos_in_buff = *data;
	    }

	    pos_in_buff++;
	    data++;
	 }

	 pos_in_buff += bwidth - (endx - startx);
	 data += width - (endx - startx);
      }
   }				/* IF */

   JLIB_PRINT_DEBUG_INFO ("Leaving buff_draw_sprite()\n");
}


/*+----------------------------------------------------------------------+ */
/*|draw_spriteNC - draw a sprite in a buff without clipping.             | */
/*+----------------------------------------------------------------------+ */

void buff_draw_spriteNC (sprite_system * spr_sys, int snum, buffer_rec * obuff)
{
   sprite_data_rec *current_sprite_data;
   UBYTE *data, *pattern;
   UBYTE width, height;
   UBYTE count1, count2;
   UBYTE *pos_in_buff, datum;
   int rlecount;


   JLIB_PRINT_DEBUG_INFO ("Entering buff_draw_spriteNC()\n");

   /* don't draw sprite if it isn't turned on */
   if (SPR_IS_ON (spr_sys, snum)) {

      pos_in_buff = B_BUFF_PTR(obuff)+
		((spr_sys->sprites[snum]->y) * B_X_SIZE (obuff))
			       + spr_sys->sprites[snum]->x;

      /* the data is the sprite_data_rec indicated by it's frame */
      current_sprite_data = spr_sys->sprite_data[spr_sys->sprites[snum]->frame];

      data = current_sprite_data->data;
      pattern = current_sprite_data->pattern;
      width = current_sprite_data->width;
      height = current_sprite_data->height;

      /* move through the RLE pattern determining which pixels to draw */
      for (count1 = 0; count1 < height; count1++) {

	 for (count2 = *pattern, ++pattern; count2 > 0; count2--) {

	    datum = *pattern;
	    ++pattern;

	    rlecount = *pattern;
	    ++pattern;

	    if (datum) {
	       FAST_SHORT_COPY (data, pos_in_buff, rlecount);
	    }

	    pos_in_buff += rlecount;
	    data += rlecount;
	 }

	 pos_in_buff += B_X_SIZE (obuff) - width;
      }
   }				/* IF */

   JLIB_PRINT_DEBUG_INFO ("Leaving buff_draw_spriteNC()\n");
}


/*+----------------------------------------------------------------------+ */
/*|save_sprite - save the buff under a sprite with clipping              | */
/*+----------------------------------------------------------------------+ */

void buff_save_sprite (sprite_system * spr_sys, int snum, buffer_rec * obuff)
{
   sprite_data_rec *current_sprite_data;
   UBYTE *data, *buffer;
   UBYTE width, height, status = NOT_CLIPPED;
   UBYTE count1, count2, startx, starty, endx, endy;
   UBYTE *pos_in_buff, datum;
   int xpos, ypos,bwidth,bheight;


   JLIB_PRINT_DEBUG_INFO ("Entering buff_save_sprite()\n");


   if((obuff == NULL)||(spr_sys==NULL)){
      jlib_exit("NULL handle passed to buff_save_sprite.");
   }
   
   bwidth  = B_X_SIZE(obuff);
   bheight = B_Y_SIZE(obuff);

   /* the data is the sprite_data_rec indicated by it's frame */
   current_sprite_data = spr_sys->sprite_data[spr_sys->sprites[snum]->frame];

      if(current_sprite_data == NULL){
         jlib_exit("NULL sprite_data_rec passed to buff_save_sprite.");
      }

   data = current_sprite_data->data;
   buffer = spr_sys->sprites[snum]->buffer;
   width = current_sprite_data->width;
   height = current_sprite_data->height;

   /* set initial values of startx , endx etc */
   startx = starty = 0;
   endx = width;
   endy = height;

   xpos = spr_sys->sprites[snum]->x;
   ypos = spr_sys->sprites[snum]->y;


   /* check for "don't save"  clipping  */
   if ((LEFT >= bwidth) || (RIGHT < 0) || (TOP >= bheight) || (BOT < 0)) {
      return;
   }

   /* check the other 4 clipping conditions */
   if (LEFT < 0) {
      /* some portion of the left side of the sprite is obscured */
      status++;
      startx = - LEFT;
   }
   else {
      if (RIGHT >= bwidth) {
	 /* some portion of the right side of the sprite is obscured */
	 status++;
	 endx = bwidth - LEFT;
      }
   }

   if (TOP < 0) {
      /* some portion of the top side of the sprite is obscured */
      status++;
      starty = - TOP;
   }
   else {
      if (BOT >= bheight) {
	 /* some portion of the bottom side of the sprite is obscured */
	 status++;
	 endy = bheight - TOP;
      }
   }

   /* only use the saving routine below if needed */
   if (status == NOT_CLIPPED) {
      buff_save_spriteNC (spr_sys, snum, obuff);
      return;
   }


   pos_in_buff = B_BUFF_PTR(obuff)+(ypos * bwidth)+xpos;

   /* start at the appropriate place in the data */
   data += (starty * width) + startx;

   /* start at the appropriate place in buff */
   pos_in_buff += (starty * bwidth) + startx;


   /* move through the data,  saving non-zero pixels */
   for (count1 = starty; count1 < endy; count1++) {
      for (count2 = startx; count2 < endx; count2++) {
	 datum = *data;

	 if (datum) {
	    *buffer = *pos_in_buff;
	 }

	 pos_in_buff++;
	 data++;
	 buffer++;
      }

      pos_in_buff += bwidth - (endx - startx);
      data += width - (endx - startx);
   }

   JLIB_PRINT_DEBUG_INFO ("Leaving buff_save_sprite()\n");
}


/*+----------------------------------------------------------------------+ */
/*|buff_save_spriteNC - save the buff under a sprite into its buffer | */
/*+----------------------------------------------------------------------+ */

void buff_save_spriteNC (sprite_system * spr_sys, int snum, buffer_rec * obuff)
{
   sprite_data_rec *current_sprite_data;
   UBYTE *buffer, *pattern;
   UBYTE width, height;
   UBYTE count1, count2;
   UBYTE *pos_in_buff, datum;
   int rlecount;


   JLIB_PRINT_DEBUG_INFO ("Entering buff_save_spriteNC()\n");

   pos_in_buff = B_BUFF_PTR(obuff) +
		((spr_sys->sprites[snum]->y) * B_X_SIZE (obuff))
			    + spr_sys->sprites[snum]->x;

   /* the data is the sprite_data_rec indicated by it's frame */
   current_sprite_data = spr_sys->sprite_data[spr_sys->sprites[snum]->frame];

   buffer = spr_sys->sprites[snum]->buffer;
   pattern = current_sprite_data->pattern;
   width = current_sprite_data->width;
   height = current_sprite_data->height;

   /* move through the RLE pattern determining which pixels to save */
   for (count1 = 0; count1 < height; count1++) {

      for (count2 = *pattern, ++pattern; count2 > 0; count2--) {

	 datum = *pattern;
	 ++pattern;

	 rlecount = *pattern;
	 ++pattern;

	 if (datum) {
	    FAST_SHORT_COPY (pos_in_buff, buffer, rlecount);
	    buffer += rlecount;
	 }
	 pos_in_buff += rlecount;

      }

      pos_in_buff += B_X_SIZE (obuff) - width;
   }

   JLIB_PRINT_DEBUG_INFO ("Leaving buff_save_spriteNC()\n");
}

/*+----------------------------------------------------------------------+ */
/*|rest_sprite - restore the buff under a sprite with clipping         | */
/*+----------------------------------------------------------------------+ */

void buff_rest_sprite (sprite_system * spr_sys, int snum, buffer_rec * obuff)
{
   sprite_data_rec *current_sprite_data;
   UBYTE *data, *buffer;
   UBYTE width, height, status = NOT_CLIPPED;
   UBYTE count1, count2, startx, starty, endx, endy;
   UBYTE *pos_in_buff, datum;
   int xpos, ypos,bwidth,bheight;


   JLIB_PRINT_DEBUG_INFO ("Entering buff_rest_sprite()\n");

   if((obuff == NULL)||(spr_sys==NULL)){
      jlib_exit("NULL handle passed to buff_rest_sprite.");
   }
   
   bwidth  = B_X_SIZE(obuff);
   bheight = B_Y_SIZE(obuff);

   /* the data is the sprite_data_rec indicated by it's frame */
   current_sprite_data = spr_sys->sprite_data[spr_sys->sprites[snum]->frame];

      if(current_sprite_data == NULL){
         jlib_exit("NULL sprite_data_rec passed to buff_rest_sprite.");
      }

   data = current_sprite_data->data;
   buffer = spr_sys->sprites[snum]->buffer;
   width = current_sprite_data->width;
   height = current_sprite_data->height;

   /* set initial values of startx , endx etc */
   startx = 0;
   starty = 0;
   endx = width;
   endy = height;

   xpos = spr_sys->sprites[snum]->x;
   ypos = spr_sys->sprites[snum]->y;


   /* check for "don't restore"  clipping  */
   if ((LEFT >= bwidth) ||
       (RIGHT < 0) ||
       (TOP > bheight) ||
       (BOT < 0)) {
      return;
   }

   /* check the other 4 clipping conditions */
   if (LEFT < 0) {
      /* some portion of the left side of the sprite is obscured */
      status++;
      startx = - LEFT;
   }
   else {
      if (RIGHT >= bwidth) {
	 /* some portion of the right side of the sprite is obscured */
	 status++;
	 endx = (bwidth) - LEFT;
      }
   }

   if (TOP < 0) {
      /* some portion of the top side of the sprite is obscured */
      status++;
      starty = - TOP;
   }
   else {
      if (BOT >= bheight) {
	 /* some portion of the bottom side of the sprite is obscured */
	 status++;
	 endy = bheight - TOP;
      }
   }

   /* only use the saving routine below if needed */
   if (status == NOT_CLIPPED) {
      buff_rest_spriteNC (spr_sys, snum, obuff);
      return;
   }


   pos_in_buff = B_BUFF_PTR(obuff)+(ypos*bwidth)+xpos;

   /* start at the appropriate place in the data */
   data += (starty * width) + startx;

   /* start at the appropriate place on buff */
   pos_in_buff += (starty * bwidth) + startx;


   /* move through the data,  saving non-zero pixels */
   for (count1 = starty; count1 < endy; count1++) {
      for (count2 = startx; count2 < endx; count2++) {
	 datum = *data;

	 if (datum) {
	    *pos_in_buff = *buffer;
	 }

	 pos_in_buff++;
	 data++;
	 buffer++;
      }

      pos_in_buff += bwidth - (endx - startx);
      data += width - (endx - startx);
   }

   JLIB_PRINT_DEBUG_INFO ("Leaving buff_rest_sprite()\n");
}


/*+----------------------------------------------------------------------+ */
/*|buff_rest_spriteNC - rest the buff under a sprite from its buffer | */
/*+----------------------------------------------------------------------+ */

void buff_rest_spriteNC (sprite_system * spr_sys, int snum, buffer_rec * obuff)
{
   sprite_data_rec *current_sprite_data;
   UBYTE *buffer, *pattern;
   UBYTE width, height;
   UBYTE count1, count2;
   UBYTE *pos_in_buff, datum;
   int rlecount;


   JLIB_PRINT_DEBUG_INFO ("Entering buff_rest_spriteNC()\n");


   /* posistion is calculated from x and y minus the maximum sprite width */
   /* and height, so that we can have sprites off the edges of the buff. */
   pos_in_buff = B_BUFF_PTR(obuff) +
		((spr_sys->sprites[snum]->y) * B_X_SIZE (obuff))
			    + spr_sys->sprites[snum]->x;


   /* the data is the sprite_data_rec indicated by it's frame */
   current_sprite_data = spr_sys->sprite_data[spr_sys->sprites[snum]->frame];

   buffer = spr_sys->sprites[snum]->buffer;
   pattern = current_sprite_data->pattern;
   width = current_sprite_data->width;
   height = current_sprite_data->height;

   /* move through the RLE pattern determining which pixels to restore */
   for (count1 = 0; count1 < height; count1++) {

      for (count2 = *pattern, ++pattern; count2 > 0; count2--) {

	 datum = *pattern;
	 ++pattern;

	 rlecount = *pattern;
	 ++pattern;

	 if (datum) {
	    FAST_SHORT_COPY (buffer, pos_in_buff, rlecount);
	    buffer += rlecount;
	 }

	 pos_in_buff += rlecount;
      }

      pos_in_buff += B_X_SIZE (obuff) - width;
   }

   JLIB_PRINT_DEBUG_INFO ("Leaving buff_rest_spriteNC()\n");
}


/*+----------------------------------------------------------------------+ */
/*|stamp a sprite frame on buff without clipping                       | */
/*|                                                                      | */
/*|"stamp" means that the background is drawn as black.                  | */
/*+----------------------------------------------------------------------+ */

JINLINE void buff_stamp_spriteNC (int x, int y, sprite_system * spr_sys,
				  int frame, buffer_rec * obuff)
{
   sprite_data_rec *current_sprite_data;
   UBYTE *data, *pattern;
   UBYTE width, height;
   UBYTE count1, count2;
   UBYTE *pos_in_buff, datum;
   int rlecount;

   JLIB_PRINT_DEBUG_INFO ("Entering buff_stamp_spriteNC()\n");

   pos_in_buff = B_BUFF_PTR(obuff)+ (y * B_X_SIZE (obuff)) + x;

   current_sprite_data = spr_sys->sprite_data[frame];

   data = current_sprite_data->data;
   pattern = current_sprite_data->pattern;
   width = current_sprite_data->width;
   height = current_sprite_data->height;

   for (count1 = 0; count1 < height; count1++) {

      for (count2 = *pattern, ++pattern; count2 > 0; count2--) {

	 datum = *pattern;
	 ++pattern;

	 rlecount = *pattern;
	 ++pattern;

	 FAST_SHORT_COPY (data, pos_in_buff, rlecount);

	 pos_in_buff += rlecount;
	 data += rlecount;
      }


      pos_in_buff += B_X_SIZE (obuff) - width;
   }

   JLIB_PRINT_DEBUG_INFO ("Leaving buff_stamp_spriteNC()\n");
}


/*+----------------------------------------------------------------------+ */
/*|stamp a sprite frame on buff with clipping                            | */
/*|                                                                      | */
/*|"stamp" means that the background is drawn as black.                  | */
/*+----------------------------------------------------------------------+ */

/* currently 'faked' as a buffer blit operation */

JINLINE void buff_stamp_sprite (int x, int y, sprite_system * spr_sys,
				    int frame, buffer_rec * obuff)
{
   sprite_data_rec *current_sprite_data;
   buffer_rec fake;      

   JLIB_PRINT_DEBUG_INFO ("Entering buff_stamp_sprite()\n");

   if((obuff == NULL)||(spr_sys==NULL)){
      jlib_exit("NULL handle passed to buff_stamp_sprite.");
   }

   current_sprite_data = spr_sys->sprite_data[frame];

   if(current_sprite_data==NULL){
      jlib_exit("NULL sprite data passed to buff_stamp_sprite.");
   }

   fake.buffer = current_sprite_data->data;
   fake.width = current_sprite_data->width;
   fake.height = current_sprite_data->height;

   buff_blit_buff_to(obuff,x,y,&fake,0,0,fake.width-1,fake.height-1);
   
   JLIB_PRINT_DEBUG_INFO ("Leaving buff_stamp_sprite()\n");
}


/*+----------------------------------------------------------------------+ */
/*|stencil a sprite frame on buff without clipping                     | */
/*|                                                                      | */
/*|"stencil" means that the background remains transparent.              | */
/*+----------------------------------------------------------------------+ */

JINLINE void buff_stencil_spriteNC (int x, int y, sprite_system * spr_sys,
				    int frame, buffer_rec * obuff)
{
   sprite_data_rec *current_sprite_data;
   UBYTE *data, *pattern;
   UBYTE width, height;
   UBYTE count1, count2;
   UBYTE *pos_in_buff, datum;
   int rlecount;

   JLIB_PRINT_DEBUG_INFO ("Entering buff_stencil_spriteNC()\n");

   pos_in_buff = B_BUFF_PTR(obuff)+ (y * B_X_SIZE (obuff)) + x;

   current_sprite_data = spr_sys->sprite_data[frame];

   data = current_sprite_data->data;
   pattern = current_sprite_data->pattern;
   width = current_sprite_data->width;
   height = current_sprite_data->height;

   for (count1 = 0; count1 < height; count1++) {

      for (count2 = *pattern, ++pattern; count2 > 0; count2--) {

	 datum = *pattern;
	 ++pattern;

	 rlecount = *pattern;
	 ++pattern;

	 if (datum) {
	    FAST_SHORT_COPY (data, pos_in_buff, rlecount);
	 }

	 pos_in_buff += rlecount;
	 data += rlecount;
      }

      pos_in_buff += B_X_SIZE (obuff) - width;
   }

   JLIB_PRINT_DEBUG_INFO ("Leaving buff_stencil_spriteNC()\n");
}

/*+----------------------------------------------------------------------+ */
/*|stencil a sprite frame on buff with clipping                          | */
/*|                                                                      | */
/*|"stencil" means that the background remains transparent.              | */
/*+----------------------------------------------------------------------+ */

/* currently 'faked' as a buffer stencil operation */

JINLINE void buff_stencil_sprite (int x, int y, sprite_system * spr_sys,
				    int frame, buffer_rec * obuff)
{
   sprite_data_rec *current_sprite_data;
   buffer_rec fake;      

   JLIB_PRINT_DEBUG_INFO ("Entering buff_stencil_sprite()\n");

   if((obuff == NULL)||(spr_sys==NULL)){
      jlib_exit("NULL handle passed to buff_stencil_sprite.");
   }

   current_sprite_data = spr_sys->sprite_data[frame];

   if(current_sprite_data==NULL){
      jlib_exit("NULL sprite data passed to buff_stencil_sprite.");
   }

   fake.buffer = current_sprite_data->data;
   fake.width = current_sprite_data->width;
   fake.height = current_sprite_data->height;

   buff_stencil_buff_to(obuff,x,y,&fake,0,0,fake.width-1,fake.height-1);
   
   JLIB_PRINT_DEBUG_INFO ("Leaving buff_stencil_sprite()\n");
}


/*+----------------------------------------------------------------------+ */
/*|buff_draw_all_sprites                                                 | */
/*+----------------------------------------------------------------------+ */
/*|draw all active sprites to the given buff.                            | */
/*+----------------------------------------------------------------------+ */

JINLINE void buff_draw_all_sprites (sprite_system * spr_sys, buffer_rec * obuff)
{
   int count;

   JLIB_PRINT_DEBUG_INFO ("Entering buff_draw_all_sprites()\n");


   for (count = 0; count < spr_sys->no_sprites; count++) {
      if (SPR_IS_ON (spr_sys, count)) {
	 buff_draw_sprite (spr_sys, count, obuff);
      }
   }

   JLIB_PRINT_DEBUG_INFO ("Leaving buff_draw_all_sprites()\n");
}


/*+----------------------------------------------------------------------+ */
/*|buff_save_all_sprites                                                 | */
/*+----------------------------------------------------------------------+ */
/*|save all active sprites to the given buff.                            | */
/*+----------------------------------------------------------------------+ */

JINLINE void buff_save_all_sprites (sprite_system * spr_sys, buffer_rec * obuff)
{
   int count;

   JLIB_PRINT_DEBUG_INFO ("Entering buff_save_all_sprites()\n");


   for (count = 0; count < spr_sys->no_sprites; count++) {
      if (SPR_IS_ON (spr_sys, count)) {
	 buff_save_sprite (spr_sys, count, obuff);
      }
   }

   JLIB_PRINT_DEBUG_INFO ("Leaving buff_save_all_sprites()\n");
}


/*+----------------------------------------------------------------------+ */
/*|buff_rest_all_sprites                                                 | */
/*+----------------------------------------------------------------------+ */
/*|rest all active sprites to the given buff.                            | */
/*+----------------------------------------------------------------------+ */

JINLINE void buff_rest_all_sprites (sprite_system * spr_sys, buffer_rec * obuff)
{
   int count;

   JLIB_PRINT_DEBUG_INFO ("Entering buff_rest_all_sprites()\n");


   for (count = 0; count < spr_sys->no_sprites; count++) {
      if (SPR_IS_ON (spr_sys, count)) {
	 buff_rest_sprite (spr_sys, count, obuff);
      }
   }

   JLIB_PRINT_DEBUG_INFO ("Leaving buff_rest_all_sprites()\n");
}
