/* btriang.c - code for drawing triangles to buffers
 *
 * Copyright (C) 1995  Jonathan Paul Griffiths.  All rights reserved.
 *
 * You may do anything with this code you wish EXCEPT sell it. You may sell
 * any software you create using this code,  but you MUST NOT charge for
 * the code itself.  See the file "jlib.doc" for detailed terms. 
 *
 */

#include <jlib.h>

/* Screen/window size: */
#define MAX_YRES 1280
#define MAX_XRES 1024

JINLINE void __trace_edge2( int x1, int y1, int x2, int y2 );
void tri_clip(int *a,int*b,int *c,int *d,buffer_rec *buff);

static int left[MAX_YRES], right[MAX_YRES];

/* Use Bresenham's line algorithm to trace an edge of the triangle. */
JINLINE void __trace_edge2( int x1, int y1, int x2, int y2 )
{
   int dx, dy, xf, yf, a, b, t, i;

   if (x1==x2 && y1==y2)
      return;

   if (x2>x1) {
      dx = x2-x1;
      xf = 1;
   }
   else {
      dx = x1-x2;
      xf = -1;
   }

   if (y2>y1) {
      dy = y2-y1;
      yf = 1;
   }
   else {
      dy = y1-y2;
      yf = -1;
   }

   if (dx>dy) {
      a = dy+dy;
      t = a-dx;
      b = t-dx;
      for (i=0;i<=dx;i++) {
         if (x1<left[y1]) {
            left[y1] = x1;
         }
         if (x1>right[y1]) {
            right[y1] = x1;
         }
         x1 += xf;
         if (t<0) {
            t += a;
         }
         else {
            t += b;
            y1 += yf;
         }
      }

   }
   else {
      a = dx+dx;
      t = a-dy;
      b = t-dy;
      for (i=0;i<=dy;i++) {
         if (x1<left[y1]) {
            left[y1] = x1;
         }
         if (x1>right[y1]) {
            right[y1] = x1;
         }
         y1 += yf;
         if (t<0) {
            t += a;
         }
         else {
            t += b;
            x1 += xf;
         }
      }
   }
}

/* Use Bresenham's line algorithm to trace an edge of the triangle. */
JINLINE void __clip_trace_edge2( int x1, int y1, int x2, int y2, int height )
{
   int dx, dy, xf, yf, a, b, t, i;

   if (x1==x2 && y1==y2)
      return;

   if (x2>x1) {
      dx = x2-x1;
      xf = 1;
   }
   else {
      dx = x1-x2;
      xf = -1;
   }

   if (y2>y1) {
      dy = y2-y1;
      yf = 1;
   }
   else {
      dy = y1-y2;
      yf = -1;
   }

   if (dx>dy) {
      a = dy+dy;
      t = a-dx;
      b = t-dx;
      for (i=0;i<=dx;i++) {
         if (x1<left[y1]) {
            if((y1>=0) && (y1<=height)){
               left[y1] = x1;
            }
         }
         if (x1>right[y1]) {
            if((y1>=0) && (y1<=height)){
               right[y1] = x1;
            }
         }
         x1 += xf;
         if (t<0) {
            t += a;
         }
         else {
            t += b;
            y1 += yf;
         }
      }

   }
   else {
      a = dx+dx;
      t = a-dy;
      b = t-dy;
      for (i=0;i<=dy;i++) {
         if (x1<left[y1]) {
            if((y1>=0) && (y1<=height)){
               left[y1] = x1;
            }
         }
         if (x1>right[y1]) {
            if((y1>=0) && (y1<=height)){
               right[y1] = x1;
            }
         }
         y1 += yf;
         if (t<0) {
            t += a;
         }
         else {
            t += b;
            x1 += xf;
         }
      }
   }
}


/* Draw a non-clipping filled triangle */
void buff_filled_triangleNC(buffer_rec *buff,int x1,int y1,int x2,int y2,
                           int x3,int y3,UBYTE col )
{
   int iy;
   int __lowest_bound,__highest_bound;

   JLIB_PRINT_DEBUG_INFO ("Entering buff_filled_triangleNC()\n");

   if (y1 > y2) {
        if (y1 > y3) {
            __highest_bound=y1;
            if(y2>y3){
                __lowest_bound=y3;
            }
            else{
                __lowest_bound=y2;
            }
        }
        else {
            __highest_bound=y3;
            if(y1>y2){
                __lowest_bound=y2;
            }
            else{
                __lowest_bound=y1;
            }
        }
    }
    else{
        if(y2>y3){
                __highest_bound=y2;
                if(y1>y3){
                        __lowest_bound=y3;
                }
                else{
                        __lowest_bound=y1;
                }
        }
        else{
                __highest_bound=y3;
                if(y1>y2){
                        __lowest_bound=y2;
                }
                else{
                        __lowest_bound=y1;
                }
        }
    }

   /* reset the minumum amount of the edge tables */
   for (iy=__lowest_bound;iy<=__highest_bound;iy++) {
      left[iy] = MAX_XRES+1;
      right[iy] = -1;
   }
      
   /* define edges */
   __trace_edge2( x1, y1, x2, y2 );
   __trace_edge2( x2, y2, x3, y3 );
   __trace_edge2( x3, y3, x1, y1 );

   /* fill horizontal spans of pixels from left[] to right[] */
   for (iy=__lowest_bound;iy<=__highest_bound;iy++) {
         buff_draw_h_lineNC(buff, left[iy], iy, right[iy], col);
   }

   JLIB_PRINT_DEBUG_INFO ("Leaving buff_filled_triangleNC()\n");
}


/* Draw a clipping filled triangle */
void buff_filled_triangle(buffer_rec *buff,int x1,int y1,int x2,int y2,
                          int x3,int y3,UBYTE col )
{
   int iy;

   JLIB_PRINT_DEBUG_INFO ("Entering buff_filled_triangle()\n");

   if(buff == NULL){
      jlib_exit("NULL buffer passed to buff_filled_triangle.");
   }

   /* reset the minumum amount of the edge tables */
   for (iy=-1;iy<=B_Y_SIZE(buff);iy++) {
      left[iy] = MAX_XRES+1;
      right[iy] = -1;
   }
      
   /* define edges,  with clipping */
   __clip_trace_edge2( x1, y1, x2, y2 ,B_MAX_Y(buff));
   __clip_trace_edge2( x2, y2, x3, y3 ,B_MAX_Y(buff));
   __clip_trace_edge2( x3, y3, x1, y1 ,B_MAX_Y(buff));

   /* fill horizontal spans of pixels from left[] to right[] */
   for (iy=0;iy<=B_MAX_Y(buff);iy++) {
         if(left[iy] < right[iy]){
            buff_draw_h_line(buff, left[iy], iy, right[iy], col);
         }
   }

   JLIB_PRINT_DEBUG_INFO ("Leaving buff_filled_triangle()\n");

}


/* non clipping hollow triangle */
void buff_draw_triangleNC(buffer_rec *buff,int x1,int y1,int x2,int y2,
                                int x3,int y3, UBYTE col)
{
 JLIB_PRINT_DEBUG_INFO ("Entering buff_draw_triangleNC()\n");

 buff_draw_lineNC(buff,x1,y1,x2,y2,col);
 buff_draw_lineNC(buff,x2,y2,x3,y3,col);
 buff_draw_lineNC(buff,x3,y3,x1,y1,col);

 JLIB_PRINT_DEBUG_INFO ("Leaving buff_draw_triangleNC()\n");
}


/* clipping hollow triangle */
void buff_draw_triangle(buffer_rec *buff,int x1,int y1,int x2,int y2,
                                int x3,int y3, UBYTE col)
{

 JLIB_PRINT_DEBUG_INFO ("Entering buff_draw_triangle()\n");

 if(buff == NULL){
    jlib_exit("NULL buffer passed to buff_draw_triangle.");
 }

 buff_draw_line(buff,x1,y1,x2,y2,col);
 buff_draw_line(buff,x2,y2,x3,y3,col);
 buff_draw_line(buff,x3,y3,x1,y1,col);

 JLIB_PRINT_DEBUG_INFO ("Leaving buff_draw_triangle()\n");
}



   

