/*
   ** jio.c - jlib device independant file input and output
   **
   ** Copyright (C) 1995  Jonathan Paul Griffiths.  All rights reserved.
   **
   ** You may do anything with this code you wish EXCEPT sell it. You may sell
   ** any software you create using this code,  but you MUST NOT charge for
   ** the code itself.  See the file "readme.jon" for more details.
 */

#include "jio.h"

/* write an elementary type (int,short,char) where size <= 4 */
int jio_write_elementary_type(FILE *fp,void *type,size_t size)
{
  switch(size){
 
     case 1:{
         UBYTE c = *(UBYTE *)type;

         fputc(c,fp);
         
         if(ferror(fp)){
            jlib_exit("File error in write_elementary_type()");
         }
         
         break;
     }
   
     case 2:{
         USHORT c = *((USHORT *)type);

         fputc((UBYTE)(c & 0xff),fp);

         fputc((UBYTE)((c >> 8) & 0xff),fp);
         
         if(ferror(fp)){
            jlib_exit("File error in write_elementary_type()");
         }

         break;
     }

     case 4:{
         _J_32_BIT c = *((_J_32_BIT*)type);

         fputc((UBYTE)(c & 0xff),fp);

         fputc((UBYTE)((c >> 8)  & 0xff),fp);
        
         fputc((UBYTE)((c >> 16) & 0xff),fp);

         fputc((UBYTE)((c >> 24)  & 0xff),fp);
         
         if(ferror(fp)){
            jlib_exit("File error in write_elementary_type()");
         }

         break;
     }
   
     default:
         /* anything other than 1,2, or 4 bytes to be written is an error */
         jlib_exit("Invalid size in write_elementary_type().\n");
  }
  return 1;
}



/* read an elementary type (int,short,char) where size <= 4 */
int jio_read_elementary_type(FILE *fp,void *type,size_t size)
{
  switch(size){
 
     case 1:{
         UBYTE c;

         c=fgetc(fp);
         
         if(ferror(fp)){
            jlib_exit("File error in read_elementary_type()\n");
         }

         *(UBYTE *)type = c;
         
	break;
     }
   
     case 2:{
         USHORT c=0;

         c += (USHORT)fgetc(fp);

         c |= (USHORT)(fgetc(fp) << 8);
         
         if(ferror(fp)){
            jlib_exit("File error in read_elementary_type()\n");
         }

         *((USHORT *)type) = c;
         break;
     }

     case 4:{
         _J_32_BIT c=0;

         c += (_J_32_BIT)fgetc(fp);

         c |= (_J_32_BIT)(fgetc(fp) << 8);

         c |= (_J_32_BIT)(fgetc(fp) << 16);

         c |= (_J_32_BIT)(fgetc(fp) << 24);

         if(ferror(fp)){
            jlib_exit("File error in read_elementary_type()\n");
         }
                  
         *((_J_32_BIT*)type) = c;

         break;
     }
   
     default:
         /* anything other than 1,2, or 4 bytes to be read is an error */
         jlib_exit("Invalid size in read_elementary_type().\n");
  }
  
  
  return 1;
}


