/*
   ** Copyright (C) 1995,1996  Jonathan Paul Griffiths.  All rights reserved.
   **
   ** You may do anything with this code you wish EXCEPT sell it. You may sell
   ** any software you create using this code,  but you MUST NOT charge for
   ** the code itself.  See the file "jlib.doc" for detailed terms. 
 */

#ifndef __JLIB_H__              /* only include this file once */

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>

/*+-----------------------------------------------------------------------+ */
/*|Target Specific Declarations                                           | */
/*+-----------------------------------------------------------------------+ */
#include <jconfig.h>
#define MOUSE_PRESENT 1
#define MOUSE_ABSENT  0
#include <jtarget.h>

/*+-----------------------------------------------------------------------+ */
/*|Types                                                                  | */
/*+-----------------------------------------------------------------------+ */
#ifdef JDEFAULT_TYPES
#define UBYTE   unsigned char
#define BYTE    signed char
#define USHORT  unsigned short int
#define SHORT   short int
#endif


/*+-----------------------------------------------------------------------+ */
/*|Sprite Functions                                                       | */
/*+-----------------------------------------------------------------------+ */
/* return codes of function 'sprite_load' */
#define COULDNT_OPEN      1
#define TOO_MANY_IN_FILE  2
#define OUT_OF_MEMORY     3
#define SUCCESS           0

/* macros to get information about a sprite system */
#define SPR_MAX_X (64)          /* biggest possible sprite x */
#define SPR_MAX_Y (64)          /* biggest possible sprite y */
#define SPR_MAX_SPRITES(sys)    ((sys)->no_sprites)
#define SPR_MAX_FRAMES(sys)     ((sys)->no_frames)
#define SPR_NUM_LOADED(sys)     ((sys)->number_loaded)
#define SPR_NUM_ACTIVE(sys)     ((sys)->no_active)

/* macros to get information about a sprite */
#define SPR_X_SIZE(spr)         ((spr)->width)
#define SPR_Y_SIZE(spr)         ((spr)->height)
#define SPR_IS_ON(sys,num)      ((sys)->active_sprites[(num)])
#define SPR_GET_XPOS(sys,num)   ((sys)->sprites[(num)]->x)
#define SPR_GET_YPOS(sys,num)   ((sys)->sprites[(num)]->y)

/* macros to set information for a sprite */
#define SPR_SET_WIDTH(spr,w)    ((spr)->width=(w))
#define SPR_SET_HEIGHT(spr,h)   ((spr)->height=(h))
#define SPR_SET_XPOS(spr,x)     ((spr)->(x)=(x))
#define SPR_SET_YPOS(spr,y)     ((spr)->(y)=(y))

/* modes for sprite animation/movement */
#define SPR_MODE_WHOLE 0        /* moving in discrete steps */
#define SPR_MODE_FIXED 1        /* moving in fixed-point steps */

/* sprite data structures */
typedef struct{
     int x;                     /* current location */
     int y;

     UBYTE mode;                /* fixed point or whole steps */

     UBYTE speed;               /* movement speed (0 = not moving ) */
     UBYTE speed_counter;       /* internal counter */
     BYTE xinc;                 /* x and y increments when moving  */
     BYTE yinc;
     
     UBYTE animspeed;           /* animation speed (0 = not animating ) */
     UBYTE animspeed_counter;   /* internal counter */
     UBYTE noframes;            /* number of animation frames */
     int frame_count;           /* internal counter */
     int frame;                 /* current frame */
     int *animframes;           /* array of frames in animation */

     UBYTE *buffer;             /* holds the information under the sprite */
} sprite_record;


typedef struct{
     UBYTE width;               /* width of sprite */
     UBYTE height;              /* height of sprite */
     UBYTE *data;               /* x*y array of the data */
     UBYTE *pattern;            /* RLE storage pattern */
     UBYTE no_rects;            /* number of bounding rectangles */
     UBYTE *rect_coords;        /* coordinates of bounding rectangles */
} sprite_data_rec;


typedef struct{
     int no_sprites;            /* max number of sprites in system     */
     UBYTE *active_sprites;     /* array of flags for active sprites   */
     int no_active;             /* number of active sprites            */
     sprite_record **sprites;   /* array of the sprites themselves     */
     int no_frames;             /* num of sprite data frames in system */
     int number_loaded;         /* number of sprite frames loaded      */
     int biggest_rle;           /* biggest rle pattern used */
     sprite_data_rec **sprite_data;     /* array of frame data         */
} sprite_system;

/* this #define makes it easier to read the function declarations */
#define SS sprite_system

/* sprite functions */
SS *sprite_init (int max_sprites, int max_frames);
SS *sprite_free (SS * ssys);

UBYTE sprite_load (char *filename, SS * ssys);
UBYTE sprite_load_fp(FILE *fp, SS * ssys);

void sprite_turn_on (SS * spr_sys, int snum);
void sprite_turn_off (SS * spr_sys, int snum);
void sprite_set_xy (SS * spr_sys, int snum, int newx, int newy);
void sprite_set_an_frame (SS * ssys, int snum, int frame);

int sprite_find_first_free (SS * spr_sys);
int sprite_find_first_frame(SS * spr_sys);

void sprite_set_mode(SS *spr_sys,int snum,int mode);
void sprite_set_move_info(SS *spr_sys,int snum,UBYTE speed,BYTE xinc,BYTE yinc);
void sprite_set_anim_info(SS *spr_sys,int snum,UBYTE anspeed,UBYTE noframes,int *animpat);

void sprite_update_anim_and_move (SS * spr_sys, int snum);
void sprite_update_all_anim_and_move (SS * spr_sys);
void sprite_do_all_anim_and_move_n_times (SS * spr_sys, int n);

int sprite_do_intersect (SS * spr_sys, int s1, int s2);



/*+-----------------------------------------------------------------------+ */
/*|Offscreen Buffer Functions                                             | */
/*+-----------------------------------------------------------------------+ */
/* macros to get information about a buffer */
#define B_X_SIZE(buff_ptr)      ((buff_ptr)->width)
#define B_Y_SIZE(buff_ptr)      ((buff_ptr)->height)
#define B_MAX_X(buff_ptr)       (((buff_ptr)->width)-1)
#define B_MAX_Y(buff_ptr)       (((buff_ptr)->height)-1)
#define B_SIZE(buff_ptr)        (((buff_ptr)->width)*((buff_ptr)->height))
#define B_BUFF_PTR(buff_ptr)    ((buff_ptr)->buffer)


/* buffer data structure */
typedef struct{
     int width;                 /* width of buffer  */
     int height;                /* height of buffer */
     UBYTE *buffer;             /* the buffer data  */
} buffer_rec;

/* this #define makes it easier to read the functions in this header */
#define BR buffer_rec

/* buffer functions */
BR *buff_init(int width, int height);
BR *buff_free(BR *buff);

void buff_clear(BR *buff);
void buff_fill(BR *buff, UBYTE colour);


void buff_blit_buff_to(BR *dbuff,int dbfx,int dbfy,BR* sbuff,int sbx1,int sby1,int sbx2,int sby2);
void buff_blit_buff_toNC(BR *dbuff,int dbfx,int dbfy,BR *sbuff,int sbx1,int sby1,int sbx2,int sby2);

void buff_stencil_buff_to (BR * dbuff, int dbfx, int dbfy,
    BR *sbuff, int sbx1, int sby1, int sbx2, int sby2);

void buff_stencil_buff_toNC (BR * dbuff, int dbfx, int dbfy,
    BR *sbuff, int sbx1, int sby1, int sbx2, int sby2);


void buff_draw_point   (BR *buff, int x, int y, UBYTE c);
void buff_draw_pointNC (BR *buff, int x, int y, UBYTE c);
UBYTE buff_get_point   (BR *buff, int x, int y);
UBYTE buff_get_pointNC (BR *buff, int x, int y);

void buff_draw_box  (BR *buff, int x1, int y1, int x2, int y2, UBYTE c);
void buff_draw_boxNC(BR *buff, int x1, int y1, int x2, int y2, UBYTE c);

void buff_draw_rect  (BR *buff, int x1, int y1, int x2, int y2, UBYTE c);
void buff_draw_rectNC(BR *buff, int x1, int y1, int x2, int y2, UBYTE c);

void buff_draw_char  (BR *buff, UBYTE letter, int x, int y, UBYTE colour);
void buff_draw_charNC(BR *buff, UBYTE letter, int x, int y, UBYTE colour);

void buff_draw_string  (BR *buff, char *string, int x, int y, UBYTE colour);
void buff_draw_stringNC(BR *buff, char *string, int x, int y, UBYTE colour);

void buff_draw_line  (BR *buff, int x, int y, int x2, int y2, UBYTE c);
void buff_draw_lineNC(BR *buff, int x, int y, int x2, int y2, UBYTE c);

void buff_draw_h_line  (BR *buff, int x1, int y1, int x2, UBYTE c);
void buff_draw_h_lineNC(BR *buff, int x1, int y1, int x2, UBYTE c);

void buff_draw_v_line  (BR *buff, int x1, int y1, int y2, UBYTE c);
void buff_draw_v_lineNC(BR *buff, int x1, int y1, int y2, UBYTE c);

void buff_draw_ellipse  (BR *buff, int x0, int y0, int a, int b, UBYTE c);
void buff_draw_ellipseNC(BR *buff, int x0, int y0, int a, int b, UBYTE c);

void buff_filled_ellipse  (BR *buff, int x0, int y0, int a, int b, UBYTE c);
void buff_filled_ellipseNC(BR *buff, int x0, int y0, int a, int b, UBYTE c);

#define buff_draw_circle(b,x0,y0,d,c) buff_draw_ellipse(b,x0,y0,d,d,c);
#define buff_draw_circleNC(b,x0,y0,d,c) buff_draw_ellipseNC(b,x0,y0,d,d,c);

#define buff_filled_circle(b,x0,y0,d,c) buff_filled_ellipse((b),(x0),(y0),(d),(d),(c));
#define buff_filled_circleNC(b,x0,y0,d,c) buff_filled_ellipseNC((b),(x0),(y0),(d),(d),(c));

void buff_draw_triangle  (BR *buff,int x1,int y1,int x2,int y2,int x3,int y3,UBYTE col);
void buff_draw_triangleNC(BR *buff,int x1,int y1,int x2,int y2,int x3,int y3,UBYTE col);

void buff_filled_triangle  (BR *buff,int x1,int y1,int x2,int y2,int x3,int y3,UBYTE col );
void buff_filled_triangleNC(BR *buff,int x1,int y1,int x2,int y2,int x3,int y3,UBYTE col);

void buff_convex_poly  (BR *buff, int n, int *x, int *y,UBYTE col);
void buff_convex_polyNC(BR *buff, int n, int *x, int *y,UBYTE col);

void buff_hollow_poly  (BR *buff, int n, int *x, int *y,UBYTE col);
void buff_hollow_polyNC(BR *buff, int n, int *x, int *y,UBYTE col);

void buff_draw_sprite     (SS *spr_sys, int snum, BR *obuff);
void buff_draw_spriteNC   (SS *spr_sys, int snum, BR *obuff);
void buff_draw_all_sprites(SS *spr_sys, BR *obuff);

void buff_save_sprite     (SS *spr_sys, int snum, BR *obuff);
void buff_save_spriteNC   (SS *spr_sys, int snum, BR *obuff);
void buff_save_all_sprites(SS *spr_sys, BR *obuff);

void buff_rest_sprite     (SS *spr_sys, int snum, BR *obuff);
void buff_rest_spriteNC   (SS *spr_sys, int snum, BR *obuff);
void buff_rest_all_sprites(SS *spr_sys, BR *obuff);

void buff_stencil_sprite  (int x, int y, SS *spr_sys, int frame, BR *obuff);
void buff_stencil_spriteNC(int x, int y, SS *spr_sys, int frame, BR *obuff);

void buff_stamp_sprite  (int x, int y, SS *spr_sys, int frame, BR *obuff);
void buff_stamp_spriteNC(int x, int y, SS *spr_sys, int frame, BR *obuff);

void sprite_build_from_buff(SS *ssys,int frame,BR *bf,int x1,int y1,int x2,int y2);

/*+-----------------------------------------------------------------------+ */
/*|Image Functions                                                        | */
/*+-----------------------------------------------------------------------+ */
#define IMG_WIDTH(img_ptr)          (img_ptr->width)
#define IMG_HEIGHT(img_ptr)         (img_ptr->height)
#define IMG_MAX_X(img_ptr)          (img_ptr->width-1)
#define IMG_MAX_Y(img_ptr)          (img_ptr->height-1)
#define IMG_PALETTE(img_ptr)        (img_ptr->palette)
#define IMG_DATA_PTR(img_ptr)       (img_ptr->data)


typedef struct
  {
     int width;
     int height;
     UBYTE *data;
     UBYTE *palette;
  }
image;

image *image_free (image *img);
image *image_load_pcx (char *filename);
image *image_load_jlb (char *filename);
int image_save_jlb(char *filename,BR *buff,UBYTE *pal); 


void buff_blit_img_to (BR *dbuff, int dbfx, int dbfy,
	   image *img, int imx1, int imy1, int imx2, int imy2);
void buff_blit_img_toNC (BR *dbuff, int dbfx, int dbfy,
	   image *img, int imx1, int imy1, int imx2, int imy2);

/*+-----------------------------------------------------------------------+ */
/*|Screen Functions                                                       | */
/*+-----------------------------------------------------------------------+ */
#ifdef JDEFAULT_SCREEN
int screen_set_video_mode(void);
void screen_restore_video_mode(void);
void screen_set_page(int page);
int screen_get_page(void);
void screen_show_page(int page);
void screen_clear(void);
void screen_fill(UBYTE colour);
void screen_wait_vsync(void);

void screen_put_pal(UBYTE col, UBYTE red, UBYTE green, UBYTE blue);

void screen_blit_buff_to(int x,int y,buffer_rec *sbuff,
			 int sbx1, int sby1,int sbx2, int sby2);
void screen_blit_buff_toNC(int x,int y,buffer_rec *sbuff,
			   int sbx1, int sby1,int sbx2, int sby2);
JINLINE void screen_blit_fs_buffer(buffer_rec *sbuff);
#endif /* ifdef JDEFAULT_SCREEN */

#ifdef JDEFAULT_LOAD_PAL
UBYTE *def_load_pal(char *fname);
#define screen_load_pal      def_load_pal
#else
#ifdef JDEFAULT_SCREEN
UBYTE *screen_load_pal(char *fname);
#endif /* default screen */
#endif /* default load pal */

#ifdef JDEFAULT_PALETTE
void def_block_set_pal(UBYTE *pal);
void def_blank_pal(void);
void def_fade_in_pal(UBYTE *pal,unsigned int delaytime);
void def_fade_out_pal(UBYTE *pal,unsigned int delaytime);

#define screen_block_set_pal def_block_set_pal
#define screen_blank_pal     def_blank_pal
#define screen_fade_in_pal   def_fade_in_pal
#define screen_fade_out_pal  def_fade_out_pal
#else
#ifdef JDEFAULT_SCREEN
void screen_block_set_pal(UBYTE *pal);
void screen_blank_pal(void);
void screen_fade_in_pal(UBYTE *pal,unsigned int delaytime);
void screen_fade_out_pal(UBYTE *pal,unsigned int delaytime);
#endif /* default screen */
#endif /* default palette */


/*+-----------------------------------------------------------------------+ */
/*|Non-Screen palette functions.                                          | */
/*+-----------------------------------------------------------------------+ */
void pal_to_grey(UBYTE *pal);
UBYTE pal_closest_color_rgb(UBYTE *pal,UBYTE r,UBYTE g,UBYTE b);


/*+-----------------------------------------------------------------------+ */
/*|Text Functions                                                         | */
/*+-----------------------------------------------------------------------+ */
#if SCREEN_WIDTH == 320
#define CHAR_WIDTH(chr)  4
#define CHAR_HEIGHT(chr) 8
#define HAS_FONT(chr)   (((UBYTE)chr>=32) && ((UBYTE)chr<=126))

#else
/* 640x480 and above modes */
#define CHAR_WIDTH(chr)  8
#define CHAR_HEIGHT(chr) 19
#define HAS_FONT(chr)   (((UBYTE)chr>=32) && ((UBYTE)chr<=126))
#endif

extern UBYTE __font_data[];


/*+-----------------------------------------------------------------------+ */
/*|Miscellanious Functions                                                | */
/*+-----------------------------------------------------------------------+ */
typedef void (*exit_function)();

extern exit_function __jlib_exit_function;

/* return library version information */
float jlib_return_version_number(void);
char *jlib_return_version_string(void);
extern char version_string[];

/* pop up a screen detailing the jlib target,  version etc */
void popup_about(UBYTE back_col,UBYTE front_col);

/* pop up a message with an OK button */
void popup_info(char *message,UBYTE back_col,UBYTE front_col);

/* set up a user defined exit function for when a library error occurs */
void jlib_set_error_handler(exit_function funct);

/* exit due to error with a message */
void jlib_exit(char *message);

/*+-----------------------------------------------------------------------+ */
/*|Debugging Functions                                                    | */
/*+-----------------------------------------------------------------------+ */
extern char JLIB_MESSAGE_STRING[81];    /* see misc/ident.c */
extern char __jlib_debug_level;         /* in misc/ident.c */

#ifdef JDEFAULT_DEBUG
#ifdef JDEBUG
#define JLIB_DEBUG_ON  {__jlib_debug_level=1;}
#define JLIB_DEBUG_OFF {__jlib_debug_level=0;}
#define JLIB_SPRINTF if(__jlib_debug_level){ sprintf
#define JLIB_SPRINTF_END }
#define JLIB_PRINT_DEBUG_INFO(x) if(__jlib_debug_level){printf(x);fflush(stdout);}
#define JLIB_PRINT_MESSAGE_STRING if(__jlib_debug_level){printf(JLIB_MESSAGE_STRING);fflush(stdout);}
#else
#define JLIB_DEBUG_ON
#define JLIB_DEBUG_OFF
#define JLIB_SPRINTF
#define JLIB_SPRINTF_END
#define JLIB_PRINT_DEBUG_INFO(x)
#define JLIB_PRINT_MESSAGE_STRING
#endif /* ifdef JDEBUG */
#endif /* ifdef JDEFAULT_DEBUG */

/*+-----------------------------------------------------------------------+ */
/*|Input Functions                                                        | */
/*+-----------------------------------------------------------------------+ */

#ifdef JDEFAULT_MOUSE
int mouse_present (void);
void mouse_show_pointer (void);
void mouse_hide_pointer (void);

void mouse_get_status (int *x_pos, int *y_pos, int *b_status);
void mouse_set_status (int x, int y);
void mouse_closedown (void);
#endif

#ifdef JDEFAULT_KB
void kb_init(void); 
void kb_closedown(void);
int kb_keydown(int key);         
int kb_key_hit(void);            
char kb_get_next_key(void);      
void kb_clear(void);             
#endif

/*+-----------------------------------------------------------------------+ */
/*|Key Definitions                                                        | */
/*+-----------------------------------------------------------------------+ */
#define KEY_ESC         1
#define KEY_1           2
#define KEY_2           3
#define KEY_3           4
#define KEY_4           5
#define KEY_5           6
#define KEY_6           7
#define KEY_7           8
#define KEY_8           9
#define KEY_9           10
#define KEY_0           11
#define KEY_MINUS       12
#define KEY_EQUALS      13
#define KEY_BACKSPACE   14
#define KEY_TAB         15
#define KEY_Q           16
#define KEY_W           17
#define KEY_E           18
#define KEY_R           19
#define KEY_T           20
#define KEY_Y           21
#define KEY_U           22
#define KEY_I           23
#define KEY_O           24
#define KEY_P           25
#define KEY_OPENBRACE   26
#define KEY_CLOSEBRACE  27
#define KEY_ENTER       28
#define KEY_CONTROL     29
#define KEY_A           30
#define KEY_S           31
#define KEY_D           32
#define KEY_F           33
#define KEY_G           34
#define KEY_H           35
#define KEY_J           36
#define KEY_K           37
#define KEY_L           38
#define KEY_COLON       39
#define KEY_QUOTE       40
#define KEY_TILDE       41
#define KEY_SHIFT       42
#define KEY_Z           44
#define KEY_X           45
#define KEY_C           46
#define KEY_V           47
#define KEY_B           48
#define KEY_N           49
#define KEY_M           50
#define KEY_COMMA       51
#define KEY_STOP        52
#define KEY_SLASH       53
#define KEY_ASTERISK    55
#define KEY_SPACE       57
#define KEY_CAPSLOCK    58
#define KEY_F1          59
#define KEY_F2          60
#define KEY_F3          61
#define KEY_F4          62
#define KEY_F5          63
#define KEY_F6          64
#define KEY_F7          65
#define KEY_F8          66
#define KEY_HOME        71
#define KEY_UP          72
#define KEY_PGUP        73
#define KEY_LEFT        75
#define KEY_RIGHT       77
#define KEY_END         79
#define KEY_DOWN        80
#define KEY_PGDN        81
#define KEY_INSERT      82
#define KEY_DEL         83


/* joystick directions */
#define JOY_NORTH     0
#define JOY_NORTHEAST 1
#define JOY_EAST      2
#define JOY_SOUTHEAST 3
#define JOY_SOUTH     4
#define JOY_SOUTHWEST 5
#define JOY_WEST      6
#define JOY_NORTHWEST 7
#define JOY_CENTER    8

#undef BR

#ifdef JLIB_SAFE_SWAP
#define JLIB_SWAP(a,b) { int temp,x,y; x=(a);y=(b);temp=x;x=y;y=temp }
#else
#define JLIB_SWAP(a,b) a^=b;b^=a;a^=b
#endif

#ifdef __cplusplus
}
#endif 

#define __JLIB_H__
#endif /* #ifndef __JLIB_H__ */
