/*
   ** Copyright (C) 1995  Jonathan Paul Griffiths.  All rights reserved.
   **
   ** You may do anything with this code you wish EXCEPT sell it. You may sell
   ** any software you create using this code,  but you MUST NOT charge for
   ** the code itself.  See the file "readme.jon" for more details.
 */

/* default palette code for targets that want to use it */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <jlib.h>


/*+------------------------------------------------------------------------+ */
/*|set a block colors                                                      | */
/*+------------------------------------------------------------------------+ */
void def_block_set_pal (UBYTE *pal)
{
   int i;

   JLIB_PRINT_DEBUG_INFO ("Entering def_block_set_pal()\n");

   for (i = 0; i < SCREEN_NUM_COLORS; i++){
	screen_put_pal (i, pal[0], pal[1], pal[2]);
	pal+=3;
   }

   JLIB_PRINT_DEBUG_INFO ("Leaving def_block_set_pal()\n");
}


/*+------------------------------------------------------------------------+ */
/*|set all colors to zero                                     | */
/*+------------------------------------------------------------------------+ */
void def_blank_pal (void)
{
   int i;

   JLIB_PRINT_DEBUG_INFO ("Entering def_blank_pal()\n");

   for (i = SCREEN_NUM_COLORS-1; i >= 0; i--){
	screen_put_pal (i, 0, 0, 0);
   }

   JLIB_PRINT_DEBUG_INFO ("Leaving def_blank_pal()\n");

}

/*+------------------------------------------------------------------------+ */
/*|load a palette from a file                                              | */
/*+------------------------------------------------------------------------+ */
#define SIZEOFPAL (SCREEN_NUM_COLORS*3)

UBYTE *def_load_pal (char *fname)
{
   UBYTE *tmp;
   FILE *fp;
   int ret;

   JLIB_PRINT_DEBUG_INFO ("Entering def_load_pal()\n");

   /* allocate memory for the palette */
   if ((tmp = (UBYTE *) malloc (SIZEOFPAL * sizeof (UBYTE))) == NULL)
     {
	JLIB_SPRINTF (JLIB_MESSAGE_STRING, "malloc failed in screen_load_pal line %d\n", __LINE__);
	JLIB_SPRINTF_END
	JLIB_PRINT_MESSAGE_STRING;

        jlib_exit("malloc() failed\n");
     }

   /* open file */
   if ((fp = fopen (fname, "rb")) == NULL){
	JLIB_SPRINTF (JLIB_MESSAGE_STRING, "Couldn't open palette file %s\n", fname);
	JLIB_SPRINTF_END
	JLIB_PRINT_MESSAGE_STRING;

	return (UBYTE *) NULL;
   }


   /* read in the palette */
   if((ret=fread(tmp,1,(sizeof (UBYTE)*SIZEOFPAL),fp))!=SIZEOFPAL){
      JLIB_SPRINTF (JLIB_MESSAGE_STRING, "Incorrect byte count read from palette file %s (read %d)\n",fname,ret);
      JLIB_SPRINTF_END
      JLIB_PRINT_MESSAGE_STRING;

      free(tmp);
      return (UBYTE *) NULL;
   }   

   JLIB_PRINT_DEBUG_INFO ("Leaving def_load_pal()\n");

   /* give if back */
   return tmp;
}

/*+------------------------------------------------------------------------+ */
/*|fade a palette in                                                       | */
/*+------------------------------------------------------------------------+ */
void def_fade_in_pal (UBYTE *pal, unsigned int delaytime)
{
   int i, j;
   UBYTE temp_pal[SIZEOFPAL];

   JLIB_PRINT_DEBUG_INFO ("Entering def_fade_in_pal()\n");

   FAST_SHORT_STORE(temp_pal, (int) 0, SIZEOFPAL);

   for (i = 1; i < 64; i++){
	for (j = 0; j < SIZEOFPAL; j++){
	     temp_pal[j] = pal[j] * i / 64;
	}
   
	screen_block_set_pal (temp_pal);
   }
   screen_block_set_pal (pal);

   JLIB_PRINT_DEBUG_INFO ("Leaving def_fade_in_pal()\n");
}


/*+------------------------------------------------------------------------+ */
/*|fade a palette out                                                      | */
/*+------------------------------------------------------------------------+ */
void def_fade_out_pal (UBYTE *pal, unsigned int delaytime)
{
   int i, j;
   UBYTE temp_pal[SIZEOFPAL];

   JLIB_PRINT_DEBUG_INFO ("Entering def_fade_out_pal()\n");

   /* copy the palette into our local copy */
   FAST_SHORT_COPY (pal, temp_pal, SIZEOFPAL);

   for (i = 64; i > 0; i--){
        for(j=0; j<SIZEOFPAL;j++){
	     temp_pal[j] = pal[j] * i / 64;
        }

	screen_block_set_pal (temp_pal);
   }
   
   screen_blank_pal();

   JLIB_PRINT_DEBUG_INFO ("Leaving def_fade_out_pal()\n");
}


#define R_COMP 19595    /* .299 << 16 */
#define G_COMP 3850     /* .587 << 16 */
#define B_COMP 9437     /* .114 << 16 */

/*+------------------------------------------------------------------------+ */
/*|convert to greyscale                                                      | */
/*+------------------------------------------------------------------------+ */
void pal_to_grey(UBYTE *pal)
{
   int i;
   unsigned long conv;

   JLIB_PRINT_DEBUG_INFO ("Entering pal_to_grey()\n");

   for(i=0;i<SCREEN_NUM_COLORS;i++){
   
       conv=(((R_COMP*pal[i*3])+(G_COMP*pal[i*3+1])+(B_COMP*pal[i*3+2]))>>16);
   
       pal[i*3]=conv;
       pal[i*3+1]=conv;
       pal[i*3+2]=conv;
   }

   JLIB_PRINT_DEBUG_INFO ("Leaving pal_to_grey()\n");
}

#ifndef ABS
#define ABS(a)   (((a) < 0) ? -(a) : (a))
#endif
#ifndef MAX
#define MAX(a,b)   (((a) < (b)) ? (b) : (a))
#endif

/*+------------------------------------------------------------------------+ */
/*|find the closest matching colour to the given rgb                                                      | */
/*+------------------------------------------------------------------------+ */
UBYTE pal_closest_color_rgb(UBYTE *pal,UBYTE r,UBYTE g,UBYTE b)
{
  int i,temp,best=256;
  UBYTE rd,gd,bd,closest=0;
  
  JLIB_PRINT_DEBUG_INFO ("Entering pal_closest_color_rgb()\n");

  for (i=SCREEN_NUM_COLORS-1;i!=0;i--){    /* skips color 0 */
    rd=ABS(r-pal[i*3]);       
    gd=ABS(g-pal[i*3+1]);
    bd=ABS(b-pal[i*3+2]);

    temp=MAX(MAX(rd,gd),bd);

    if (temp<best){
        best=temp;
        closest=i;
    }
  }

  JLIB_PRINT_DEBUG_INFO ("Leaving pal_closest_color_rgb()\n");
 
  return closest;
}

