/*
   ** Copyright (C) 1995  Jonathan Paul Griffiths.  All rights reserved.
   **
   ** You may do anything with this code you wish EXCEPT sell it. You may sell
   ** any software you create using this code,  but you MUST NOT charge for
   ** the code itself.  See the file "jlib.doc" for detailed terms. 
 */

/* target specific includes for DJGPP/MODE_13H */

#ifndef __JTARGET_H__
#define __JTARGET_H__

#ifdef __cplusplus
extern "C" {
#endif 

#ifndef DJGPP_V2
#error Wrong Target header file included in build.
#endif

#ifndef MODE_13H
#error Wrong Target header file included in build.
#endif

#include <string.h>

/* INPUT */
#define KEYBOARD_AVAILABLE 1    /* keyboard support is available */
#define MOUSE_AVAILABLE 1	/* mouse is available */ 
#define JOYSTICK_AVAILABLE 0    /* no joystick support */

/* mouse */
#define BUTTON_DOWN(status)  (status & 0x1)

/* SCREEN */
extern int _jlib_dos_selector;  /* global: in misc.c */
void jdosmemsetup(void);        /* set up global selector */

#define  SCREEN_WIDTH   320
#define  SCREEN_HEIGHT  200
#define  SCREEN_MAX_X   319
#define  SCREEN_MAX_Y   199
#define  SCREEN_NUM_COLORS 256
#define  SCREEN_NUM_PAGES 1

/* MEMORY OPERATIONS */

/* return the best copy method for two addresses and a length */
JINLINE int get_best_copy_method (void *addr1, void *addr2, unsigned int len);

#define ALIGNMENT_STATE_1   0
#define ALIGNMENT_STATE_2   1
#define ALIGNMENT_STATE_3   2
#define ALIGNMENT_STATE_4   3
#define ALIGNMENT_STATE_5   4
#define ALIGNMENT_STATE_6   5
#define ALIGNMENT_STATE_7   6
#define ALIGNMENT_STATE_8   7

/* some copies are optimised according to alignment */

#define MEM_COPY_ALIGN_1(x,y,z)  _copy_sb( (x) , (y) , (z) )
#define MEM_COPY_ALIGN_2(x,y,z)  _copy_inc_sw( (x) , (y) , (z) )
#define MEM_COPY_ALIGN_3(x,y,z)  _copy_sw( (x) , (y) , (z) )
#define MEM_COPY_ALIGN_4(x,y,z)  _copy_sw_inc( (x) , (y) , (z) )
#define MEM_COPY_ALIGN_5(x,y,z)  _copy_inc_sw_inc( (x) , (y) , (z) )
#define MEM_COPY_ALIGN_6(x,y,z)  _copy_sb( (x) , (y) , (z) )
#define MEM_COPY_ALIGN_7(x,y,z)  _copy_sb( (x) , (y) , (z) )
#define MEM_COPY_ALIGN_8(x,y,z)  _copy_sb( (x) , (y) , (z) )

#define FAST_SHORT_COPY(a,b,len) _copy_sb((a),(b),(len))
#define FAST_LONG_COPY(a,b,len)  _blast((a),(b),(len))
#define FAST_VERY_LONG_COPY(a,b,len)  _blast((a),(b),(len))
#define BYTE_COPY(a,b,len)       _bcopy((a),(b),(len))
#define WORD_COPY(a,b,len)       _copy_sw((a),(b),(len))
#define LONG_COPY(a,b,len)       _blast((a),(b),(len))

/* every memory store is done using memset */
#define MEM_STORE_ALIGN_1(d,val,len)  memset( (d) , (val) , (len) )
#define MEM_STORE_ALIGN_2(d,val,len)  memset( (d) , (val) , (len) )
#define MEM_STORE_ALIGN_3(d,val,len)  memset( (d) , (val) , (len) )
#define MEM_STORE_ALIGN_4(d,val,len)  memset( (d) , (val) , (len) )
#define MEM_STORE_ALIGN_5(d,val,len)  memset( (d) , (val) , (len) )
#define MEM_STORE_ALIGN_6(d,val,len)  memset( (d) , (val) , (len) )
#define MEM_STORE_ALIGN_7(d,val,len)  memset( (d) , (val) , (len) )
#define MEM_STORE_ALIGN_8(d,val,len)  memset( (d) , (val) , (len) )

#define FAST_SHORT_STORE(d,val,len)  memset( (d) , (val) , (len) )
#define FAST_LONG_STORE(d,val,len)   memset( (d) , (val) , (len) )
#define FAST_VERY_LONG_STORE(d,val,len) memset( (d) , (val) , (len) )
#define BYTE_STORE(d,val,len)        memset( (d) , (val) , (len) )
#define WORD_STORE(d,val,len)        memset( (d) , (val) , (len) )
#define LONG_STORE(d,val,len)        memset( (d) , (val) , (len) )


/* other copying routines */
void _blast (void *src, void *dest, unsigned int len);


/* ========================================================================
   Try to ignore this mess,  I had to this because the compiler wouldn't   
   inline the assembly.                                                            
   ========================================================================
 */

/* all of these copies are source, dest, len */
#define _copy_sb(src,dest,len) asm volatile(                              "\n\
push %%ecx \n \
movl %0, %%esi \n \
movl %1, %%edi \n \
movl %2, %%ecx \n \
rep \n \
movsb \n \
pop %%ecx "\
:                               /* NOTHING */ \
:" g " (src), " g " (dest), " g " (len) \
:"di", "si", "cx")

#define _copy_sw(src,dest,len) asm volatile(                              "\n\
push %%ecx \n \
movl %0, %%esi \n \
movl %1, %%edi \n \
movl %2, %%ecx \n \
shrl $1, %%ecx \n \
rep \n \
movsw \n \
pop %%ecx "\
:                               /* NOTHING */ \
:" g " (src), " g " (dest), " g " (len) \
:"di", "si", "cx")

#define _copy_sw_inc(src,dest,len) asm volatile(                         "\n\
push %%ecx \n \
movl %0, %%esi \n \
movl %1, %%edi \n \
movl %2, %%ecx \n \
shrl $1, %%ecx \n \
rep \n \
movsw \n \
movl $1, %%ecx \n \
rep \n \
movsb \n \
pop %%ecx "\
:                               /* NOTHING */ \
:" g " (src), " g " (dest), " g " (len) \
:"di", "si", "cx")

#define _copy_inc_sw(src,dest,len) asm volatile(                         "\n\
push %%ecx \n \
movl %0, %%esi \n \
movl %1, %%edi \n \
movl $1, %%ecx \n \
rep \n \
movsb \n \
movl %2, %%ecx \n \
shrl $1, %%ecx \n \
rep \n \
movsw \n \
pop %%ecx "\
:                               /* NOTHING */ \
:" g " (src), " g " (dest), " g " (len) \
:"di", "si", "cx")

#define _copy_inc_sw_inc(src,dest,len) asm volatile(                     "\n\
push %%ecx \n \
movl %0, %%esi \n \
movl %1, %%edi \n \
movl $1, %%ecx \n \
rep \n \
movsb \n \
movl %2, %%ecx \n \
shrl $1, %%ecx \n \
rep \n \
movsw \n \
movl $1, %%ecx \n \
rep \n \
movsb \n \
pop %%ecx "\
:                               /* NOTHING */ \
:" g " (src), " g " (dest), " g " (len) \
:"di", "si", "cx")

/* all of this stuff deals with *dos* memory copies */
#define DOS_MEMCOPY_ALIGN_1(x,y,z)  _dos_copy_sb( (x) , (y) , (z) )
#define DOS_MEMCOPY_ALIGN_2(x,y,z)  _dos_copy_inc_sw( (x) , (y) , (z) )
#define DOS_MEMCOPY_ALIGN_3(x,y,z)  _dos_copy_sw( (x) , (y) , (z) )
#define DOS_MEMCOPY_ALIGN_4(x,y,z)  _dos_copy_sw_inc( (x) , (y) , (z) )
#define DOS_MEMCOPY_ALIGN_5(x,y,z)  _dos_copy_inc_sw_inc( (x) , (y) , (z) )
#define DOS_MEMCOPY_ALIGN_6(x,y,z)  _dos_copy_sb( (x) , (y) , (z) )
#define DOS_MEMCOPY_ALIGN_7(x,y,z)  _dos_copy_sb( (x) , (y) , (z) )
#define DOS_MEMCOPY_ALIGN_8(x,y,z)  _dos_copy_sb( (x) , (y) , (z) )

#define _dos_copy_sb(src,dest,len) asm volatile(                         "\n\
pushw %%es \n \
movw __jlib_sel, %%es \n \
 push %%ecx \n \
 movl %0, %%esi \n \
 addl __jlib_off, %%esi \n \
 movl %1, %%edi \n \
 movl %2, %%ecx \n \
 rep \n \
 movsb \n \
 pop %%ecx \n \
 popw %%es "\
:                               /* NOTHING */ \
:" g " (src), " g " (dest), " g " (len) \
:"di", "si", "cx")

#define _dos_copy_sw(src,dest,len) asm volatile(                         "\n\
pushw %%es \n \
movw __jlib_sel, %%es \n \
 push %%ecx \n \
 movl %0, %%esi \n \
 addl __jlib_off, %%esi \n \
 movl %1, %%edi \n \
 movl %2, %%ecx \n \
 shrl $1, %%ecx \n \
 rep \n \
 movsw \n \
 pop %%ecx \n \
 popw %%es "\
:                               /* NOTHING */ \
:" g " (src), " g " (dest), " g " (len) \
:"di", "si", "cx")

#define _dos_copy_sw_inc(src,dest,len) asm volatile(                     "\n\
pushw %%es \n \
movw __jlib_sel, %%es \n \
 push %%ecx \n \
 movl %0, %%esi \n \
 addl __jlib_off, %%esi \n \
 movl %1, %%edi \n \
 movl %2, %%ecx \n \
 shrl $1, %%ecx \n \
 rep \n \
 movsw \n \
 movl $1, %%ecx \n \
 rep \n \
 movsb \n \
 pop %%ecx \n \
 popw %%es "\
:                               /* NOTHING */ \
:" g " (src), " g " (dest), " g " (len) \
:"di", "si", "cx")

#define _dos_copy_inc_sw(src,dest,len) asm volatile(                     "\n\
pushw %%es \n \
movw __jlib_sel, %%es \n \
 push %%ecx \n \
 movl %0, %%esi \n \
 addl __jlib_off, %%esi \n \
 movl %1, %%edi \n \
 movl $1, %%ecx \n \
 rep \n \
 movsb \n \
 movl %2, %%ecx \n \
 shrl $1, %%ecx \n \
 rep \n \
 movsw \n \
 pop %%ecx \n \
 popw %%es "\
:                               /* NOTHING */ \
:" g " (src), " g " (dest), " g " (len) \
:"di", "si", "cx")

#define _dos_copy_inc_sw_inc(src,dest,len) asm volatile(                     "\n\
pushw %%es \n \
movw __jlib_sel, %%es \n \
 push %%ecx \n \
 movl %0, %%esi \n \
 addl __jlib_off, %%esi \n \
 movl %1, %%edi \n \
 movl $1, %%ecx \n \
 rep \n \
 movsb \n \
 movl %2, %%ecx \n \
 shrl $1, %%ecx \n \
 rep \n \
 movsw \n \
 movl $1, %%ecx \n \
 rep \n \
 movsb \n \
 pop %%ecx \n \
 popw %%es "\
:                               /* NOTHING */ \
:" g " (src), " g " (dest), " g " (len) \
:"di", "si", "cx")


#ifdef __cplusplus
}
#endif 

#endif /* #ifndef __JTARGET_H__ */
