/*
   ** Copyright (C) 1995  Jonathan Paul Griffiths.  All rights reserved.
   **
   ** You may do anything with this code you wish EXCEPT sell it. You may sell
   ** any software you create using this code,  but you MUST NOT charge for
   ** the code itself.  See the file "jlib.doc" for detailed terms. 
 */

#include <jlib.h>
#include <stdio.h>
#include <dos.h>
#include <ctype.h>

#define NR_KEYS 128

static unsigned char jstate[NR_KEYS];   
static char __jlib_kb_buff[256];        /* keyboard buffer */
static unsigned char kb_head,kb_tail;   /* indices into buffers */
static int kb_initialised = 0;
static UBYTE kb_capslock_on = 0;

/* array of scan codes -> ascii values */
static char __jlib_sc_to_asc[128] =
{  0, 27, '1', '2', '3', '4', '5', '6', '7', '8', '9', '0',
   '-', '=', 8, 9, 'q', 'w', 'e', 'r', 't', 'y', 'u', 'i', 'o', 'p',
   '[', ']', 13, 0, 'a', 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l',
   ';', '\'', '`', 0, 0, 'z', 'x', 'c', 'v', 'b', 'n', 'm',
   ',', '.', '/', 0, '*', 0, 32, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, '-', 0, '5', 0, '+', 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};
static char __jlib_upp_sc_to_asc[128] =
{  0, 27, '!', '@', '#', '$', '%', '^', '&', '*', '(', ')',
   '_', '+', 8, 9, 'Q', 'W', 'E', 'R', 'T', 'Y', 'U', 'I', 'O', 'P',
   '{', '}', 13, 0, 'A', 'S', 'D', 'F', 'G', 'H', 'J', 'K', 'L',
   ':', '|', '~', 0, 0, 'Z', 'X', 'C', 'V', 'V', 'B', 'M',
   '<', '>', '?', 0, '*', 0, 32, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, '-', 0, '5', 0, '+', 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static char __jlib_caps_sc_to_asc[128] =
{
/* 0   1   2   3   4   5   6   7   8   9   A   B   C   D   E   F           */
   0,  27, '1','2','3','4','5','6','7','8','9','0','-','=',8,  9,     /* 0 */
   'Q','W','E','R','T','Y','U','I','O','P','[',']',13, 0,  'A','S',   /* 1 */
   'D','F','G','H','J','K','L',';',39, '`',0,  92, 'Z','X','C','V',   /* 2 */
   'B','N','M',',','.','/',0,  '*',0,  ' ',0,  0,  0,  0,  0,  0,     /* 3 */
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  '-',0,  0,  0,  '+',0,     /* 4 */
   0,  0,  0,  127,0,  0,  92, 0,  0,  0,  0,  0,  0,  0,  0,  0,     /* 5 */
   13, 0,  '/',0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  127,   /* 6 */
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0      /* 7 */
};

void (__interrupt __far *oldkeyhandler)();

void __interrupt __far newkeyhandler()
{
    int scancode; 
    int newstate;

    scancode = (inp(0x60)&0xff);
    newstate=1-(scancode >>7);
    scancode=scancode&127;

	if(scancode < 0 || scancode >= NR_KEYS) {
           /* invalid scancode - ignore */
           return;
	}

  	/* set state to reflect newstate */	
	jstate[scancode] = newstate;

        if((scancode==58)&&(newstate==0)){
           /* caps lock released */
           if(kb_capslock_on==0){
              kb_capslock_on=1;
           }
           else{
              kb_capslock_on=0;
           }
        }

	/* add key to input buffer if apropriate */
	if(newstate != 0){
	
	   /* the key is being pressed,  not released... */
	   if(__jlib_sc_to_asc[scancode]!=0){
	   
	      /* and it is an ascii char,  so add it to the buffer */
              kb_tail++; /* will wrap automatically */
              
              if((jstate[54])||(jstate[42])){
                 __jlib_kb_buff[kb_head] = __jlib_upp_sc_to_asc[scancode];
              }
              else{
                  if(kb_capslock_on){
                     __jlib_kb_buff[kb_head] = __jlib_caps_sc_to_asc[scancode];
                  }
                  else{
                     __jlib_kb_buff[kb_head] = __jlib_sc_to_asc[scancode];
                  }
              }
              if(kb_tail == kb_head){
                 kb_head++;
              }	        
           }           
        }
    
    outp(0x20,0x20);
}


void kb_init(void)
{
 if(kb_initialised){
    return;
 }
 
 kb_initialised = 1;
 memset(jstate,0,NR_KEYS); /* clear key states */
 kb_head = kb_tail = 0;
 
 oldkeyhandler = _dos_getvect(0x9); 
 _dos_setvect(0x9, newkeyhandler);
}

void kb_closedown(void)
{
 if(kb_initialised == 0){
    return;
 }
 
 kb_initialised = 0;

   _dos_setvect(9, oldkeyhandler); 
} 




void kb_clear(void)
{
  if(kb_initialised == 0){
    return;
  }
 
 memset(jstate,0,NR_KEYS); /* clear key states */
 
 kb_head = kb_tail = 0; /* clear keyboard input buffer */
 
} 


int kb_keydown(int key)
{
  if(kb_initialised == 0){
    return 0;
  }

 if(key < 0 || key >= NR_KEYS) {
    return 0;
 }

 return jstate[key]; 
}


int kb_key_hit(void)
{
  if(kb_initialised == 0){
    return 0;
  }

 return (kb_head!=kb_tail);    
}


char kb_get_next_key(void)
{
 
 char temp;
 
  if(kb_initialised == 0){
    return 0;
  }

 /* loop until a key is pressed,  or fall thru if one is waiting */
 while(kb_head == kb_tail){
    /* do nothing */
 }
 
 temp = __jlib_kb_buff[kb_head];    
 
 kb_head++;     /* remove the char from the buffer */
     
 return temp;
}

